/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/


// a class to handle database errors

#ifndef __DATA_ERROR_H__
#define __DATA_ERROR_H__

#include <string>
#include <stdexcept>

#include "../ibpp/ibpp.h"

using namespace std;

class CDataErrorException : public runtime_error {

public:

	/// cause of the error
	enum ErrorCause {
		ecUnspecified, /*!< no known cause, used when converting from IBPP exceptions */
		ecUnique, /*!< unique constraint violated */
		ecReferentialIntegrity, /*!< referential integrity violated */
		ecCheck, /*!< check constraint violated */
		ecException, /*!< exception raised from the database */
		ecServerNotFound, /*!< server not found */
		ecDatabaseNotFound, /*!< database not found */
		ecWrongUsernameOrPassword /*!< wrong username or password */
	};

	CDataErrorException( const string &errMsg, ErrorCause ec );

	CDataErrorException( const char *errMsg, ErrorCause ec );

	/// return the cause of this error
	ErrorCause GetErrorCause();

	/// convert an error from Firebird error code.
	/// returns true if the error code is handled by this class and sets the "ec" parameter to the correct value
	/// returns false and does not change "ec" if the error code is not handled.
	static bool ConvertFirebirdError( long gdscode, ErrorCause& ec );

	/// convert an IBPP::LogicException to a CDataErrorException
    /// addtionalInfo can contain more information that will be added to the exception description
    static CDataErrorException IBPPLogic2CDataError( IBPP::LogicException &eIn, const string &additionalInfo = "" );

	/// convert an IBPP::LogicException to a CDataSqlException
	static CDataErrorException IBPPSql2CDataError( IBPP::SQLException &eIn );

private:
	
	/// the cause of this error
	ErrorCause m_ErrorCode;

};

#endif

