/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/

/**
Abstract class that defines a database, intended as the information needed to connect to it
*/

#ifndef __FSDB_DATABASE_H__
#define __FSDB_DATABASE_H__

#include "recordset_base.h"
#include "parameters.h"

#include <string>
#include <vector>

namespace FSDB {

	/** Abstract base class for a database.
	Defines the common interface of all database objects.
	*/
	class DatabaseBase {

	public:
	
        virtual ~DatabaseBase() {}

		/// Connect to the database. 
		virtual void Connect() = 0;
		/// Disconnect from the database
		virtual void Disconnect() = 0;

        // return true if this instance is connected to a database
        virtual bool IsConnected() = 0;

		/// start a transaction, intended as an atomic sequence of write operation
		virtual void TransactionStart() = 0;
		virtual void TransactionCommit() = 0;
		virtual void TransactionRollback() = 0;

		/// create a recordset object
		virtual RecordsetPt CreateRecordset( RecordsetType rt ) = 0;

		/// convert a variant to a string to be used in SQL commands.
		/// the string might change with different databases
		virtual std::string ConvertToSQLString( const FSDB::Variant& var ) = 0;

		/// execute a database command that does not return anything.
		virtual void ExecuteNoReturn( const std::string& sql ) = 0;
		virtual void ExecuteNoReturn( const std::string& sql, Parameters params ) = 0;

		/// execute a database command that returns an integer value
		virtual int ExecuteReturnInt( const std::string& sql ) = 0;
		virtual int ExecuteReturnInt( const std::string& sql, Parameters params ) = 0;

		/// execute a database command that returns a string value
		virtual std::string ExecuteReturnString( const std::string& sql ) = 0;
		virtual std::string ExecuteReturnString( const std::string& sql, Parameters params ) = 0;

		/// update the statistics for the tables received in the vector of strings.
		/// some databases need this command after massive changes to make the optimizer works well
		/// if a database does not need this function it will not do anything
		virtual void UpdateStatistics( const std::vector<std::string> &tables ) = 0;

		/// backup the current database
		virtual void Backup( const std::string &backupName ) = 0;
	};

	/// the type name used by programs for a pointer to a database object
	typedef boost::shared_ptr<DatabaseBase> Database;
}


#endif

