/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/

#include "database_firebird.h"
#include "db_interface_firebird.h"
#include "recordset_rfo_ro.h"
#include "recordset_fb_rw.h"
#include "data_error.h"

#include <iostream>
#include <sstream>
#include <stdexcept>
#include <assert.h>
#include <iomanip>

FSDB::DatabaseFirebird::DatabaseFirebird( const std::string &serverName, const std::string &databaseName, 
                                          const std::string &userName, const std::string &password ) {

	this->serverName = serverName;
	this->databaseName = databaseName;
	this->userName = userName;
	this->password = password;

	transactionPending = false;

	db = IBPP::DatabaseFactory( serverName, databaseName, userName, password );
}

void FSDB::DatabaseFirebird::Connect() {
	try {
		db->Connect();
	}
	catch( IBPP::SQLException& e ) {
		throw CDataErrorException::IBPPSql2CDataError( e );
	}
	catch( IBPP::LogicException& e ) {
		throw CDataErrorException::IBPPLogic2CDataError( e );
	}
	catch(...) {
		throw;
	}
}


void FSDB::DatabaseFirebird::Disconnect() {
	db->Disconnect();
}

bool FSDB::DatabaseFirebird::IsConnected() {
    return db->Connected();
}

void FSDB::DatabaseFirebird::TransactionStart() {
	
	if( transactionPending ) {
		throw std::logic_error( "A transaction is already started" );
	}

	transactionPending = true;
	tr = IBPP::TransactionFactory( db, IBPP::amWrite );
	tr->Start();
}

void FSDB::DatabaseFirebird::TransactionCommit() {

	if( !transactionPending ) {
		throw std::logic_error( "No pending transaction to commit" );
	}

	transactionPending = false;
	tr->Commit();
}

void FSDB::DatabaseFirebird::TransactionRollback() {

	if( !transactionPending ) {
		throw std::logic_error( "No pending transaction to rollback" );
	}

	transactionPending = false;
	tr->Rollback();
}

FSDB::RecordsetPt FSDB::DatabaseFirebird::CreateRecordset( RecordsetType rt ) {

	FSDB::DbInterfaceFirebird* dbi = new FSDB::DbInterfaceFirebird( this );
	FSDB::RecordsetPt rs;

	switch( rt ) {
		case rtReadOnlyForwardOnly:
			{
				boost::shared_ptr<RecordsetBase> p(new FSDB::Recordset_FO_RO(rt, dbi) );
				rs = p;
			}
			break;
		case rtReadOnlyRepeatableForwardOnly:
			{
				boost::shared_ptr<RecordsetBase> p(new FSDB::Recordset_RFO_RO(rt, dbi) );
				rs = p;
			}
			break;
		case rtReadWriteForwardOnly:
			{
				boost::shared_ptr<RecordsetBase> p(new FSDB::Recordset_FO_RW(rt, dbi) );
				rs = p;
			}
			break;
		case rtReadWrite:
			{
				boost::shared_ptr<RecordsetBase> p(new FSDB::Recordset_FB_RW(rt, dbi) );
				rs = p;
			}
			break;
		default:
			throw std::invalid_argument( "Unhandled recordset type" );
			break;
	}

	return rs;
}

std::string FSDB::DatabaseFirebird::ConvertToSQLString( const FSDB::Variant& var ) {
	std::ostringstream o;
	std::string s;

	switch( var.GetCurrentType() ) {
		case FSDB::Variant::ctString:
			// escapes "'" characters
			s = var.GetAsString();
			{
				size_t lookHere = 0;
				size_t foundHere;
				while( (foundHere = s.find( "'", lookHere )) != std::string::npos ) {
					s.replace( foundHere, 1, "''" );
					lookHere = foundHere + 2;
				}
			}
			s = "'" + s + "'";
			break;
		case FSDB::Variant::ctFloat:
		case FSDB::Variant::ctDouble:
			if( !(o << setprecision(16) << var.GetAsDouble()) )
				throw std::runtime_error( "Conversion error: ConvertToSQLString(double)" );
			s = o.str();
			break;
		case FSDB::Variant::ctSmallInt:
		case FSDB::Variant::ctInteger:
		case FSDB::Variant::ctLargeInt:
			if( !(o << var.GetAsLargeInt()) )
				throw std::runtime_error( "Conversion error: ConvertToSQLString(largeint)" );
			s = o.str();
			break;
		case FSDB::Variant::ctTimestamp:
			s = "'" + Int2String( var.GetAsTimestamp().Year(), 4 ) + "-" + Int2String( var.GetAsTimestamp().Month(), 2 ) +
				"-" + Int2String( var.GetAsTimestamp().Day(), 2 ) + " " + Int2String( var.GetAsTimestamp().Hours(), 2 ) +
				":" + Int2String( var.GetAsTimestamp().Minutes(), 2 ) + ":" + Int2String( var.GetAsTimestamp().Seconds() , 2 ) +
				"." + Int2String( var.GetAsTimestamp().SubSeconds(), 4 ) + "'";
			break;
		case FSDB::Variant::ctBlob:
			s = "?";
			break;
		case FSDB::Variant::ctNull:
			s = "NULL";
			break;
		default:
			throw std::runtime_error( "ConvertToSQLString(unhandled type)" );
			break;
	}

	return s;
}

std::string FSDB::DatabaseFirebird::Int2String( int32_t i, int nChars ) {
	std::ostringstream o;
	std::string s;

	if( !(o << i ) )
		throw std::runtime_error( "Int2String() conversion error" );
	s = o.str();

	if( s.length() < (unsigned int) nChars ) {
		std::string z( nChars - s.length(), '0' );
		s = z + s;
	}

	return s;
}

void FSDB::DatabaseFirebird::ExecuteNoReturn( const std::string& sql ) {
	ExecuteNoReturn( sql, ParametersFactory() );
}

void FSDB::DatabaseFirebird::ExecuteNoReturn( const std::string& sql, Parameters params ) {
	IBPP::Transaction m_tr;
	IBPP::Statement m_st;

	assert( sql != "" );

	if( !IsTransactionPending() ) {
		m_tr = TransactionFactory( GetIBPPDatabase(), IBPP::amWrite );
		m_tr->Start();
	}
	else {
		m_tr = GetIBPPTransaction();
	}

	m_st = StatementFactory( GetIBPPDatabase(), m_tr );
	try {
		if( params->Size() == 0 ) {
			m_st->Execute( sql );
		}
		else {
			m_st->Prepare( sql );
			SetStatementParameters( m_st, params );
			m_st->Execute();
		}
	}
	catch( IBPP::SQLException& e ) {
		throw CDataErrorException::IBPPSql2CDataError( e );
	}
	catch( IBPP::LogicException& e ) {
		throw CDataErrorException::IBPPLogic2CDataError( e );
	}
	catch(...) {
		throw;
	}

	if( !IsTransactionPending() ) {
		m_tr->Commit();
	}
}

int FSDB::DatabaseFirebird::ExecuteReturnInt( const std::string& sql ) {
	return ExecuteReturnInt( sql, ParametersFactory() );
}

int FSDB::DatabaseFirebird::ExecuteReturnInt( const std::string& sql, Parameters params ) {
	IBPP::Transaction m_tr;
	IBPP::Statement m_st;

	assert( sql != "" );

	if( !IsTransactionPending() ) {
		m_tr = TransactionFactory( GetIBPPDatabase(), IBPP::amWrite );
		m_tr->Start();
	}
	else {
		m_tr = GetIBPPTransaction();
	}

	m_st = StatementFactory( GetIBPPDatabase(), m_tr );
	try {
		if( params->Size() == 0 ) {
			m_st->Execute( sql );
		}
		else {
			m_st->Prepare( sql );
			SetStatementParameters( m_st, params );
			m_st->Execute();
		}
	}
	catch( IBPP::SQLException& e ) {
		throw CDataErrorException::IBPPSql2CDataError( e );
	}
	catch( IBPP::LogicException& e ) {
		throw CDataErrorException::IBPPLogic2CDataError( e );
	}
	catch(...) {
		throw;
	}

	// get the returned value
    if( m_st->Type() != IBPP::stExecProcedure ) {
        // a stored procedure returning a single value does not require the call to Fetch()
        bool r = m_st->Fetch();
        assert( r );
    }
    int64_t tmp;
	m_st->Get( 1, tmp );

	if( !IsTransactionPending() ) {
		m_tr->Commit();
	}

	return (int) tmp;
}

std::string FSDB::DatabaseFirebird::ExecuteReturnString( const std::string& sql ) {
	return ExecuteReturnString( sql, ParametersFactory() );
}

std::string FSDB::DatabaseFirebird::ExecuteReturnString( const std::string& sql, Parameters params ) {
	IBPP::Transaction m_tr;
	IBPP::Statement m_st;

	assert( sql != "" );

	if( !IsTransactionPending() ) {
		m_tr = TransactionFactory( GetIBPPDatabase(), IBPP::amWrite );
		m_tr->Start();
	}
	else {
		m_tr = GetIBPPTransaction();
	}

	m_st = StatementFactory( GetIBPPDatabase(), m_tr );
	try {
		if( params->Size() == 0 ) {
			m_st->Execute( sql );
		}
		else {
			m_st->Prepare( sql );
			SetStatementParameters( m_st, params );
			m_st->Execute();
		}
	}
	catch( IBPP::SQLException& e ) {
		throw CDataErrorException::IBPPSql2CDataError( e );
	}
	catch( IBPP::LogicException& e ) {
		throw CDataErrorException::IBPPLogic2CDataError( e );
	}
	catch(...) {
		throw;
	}

	// get the returned value
    if( m_st->Type() != IBPP::stExecProcedure ) {
        // a stored procedure returning a single value does not require the call to Fetch()
        bool r = m_st->Fetch();
        assert( r );
    }
	std::string tmp;
	m_st->Get( 1, tmp );

	if( !IsTransactionPending() ) {
		m_tr->Commit();
	}

	return tmp;
}

void FSDB::DatabaseFirebird::SetStatementParameters( IBPP::Statement st, Parameters params ) {
	ParametersImpl& p = *params;
	for( unsigned int k = 0; k < p.Size(); k++ ) {
		switch( p[k].GetCurrentType() ) {
			case Variant::ctNull:
				st->SetNull( k + 1 );
				break;
			case Variant::ctString:
				st->Set( k + 1, p[k].GetAsString() );
				break;
			case Variant::ctFloat:
				st->Set( k + 1, p[k].GetAsFloat() );
				break;
			case Variant::ctDouble:
				st->Set( k + 1, p[k].GetAsDouble() );
				break;
			case Variant::ctSmallInt:
				st->Set( k + 1, p[k].GetAsSmallInt() );
				break;
			case Variant::ctInteger:
				st->Set( k + 1, p[k].GetAsInteger() );
				break;
			case Variant::ctLargeInt:
				st->Set( k + 1, p[k].GetAsLargeInt() );
				break;
			case Variant::ctTimestamp:
				FSDB::Timestamp ts = p[k].GetAsTimestamp();
				// convert to IBPP::Timestamp
				IBPP::Timestamp IBPPts( ts.Year(), ts.Month(), ts.Day(), ts.Hours(), ts.Minutes(), ts.Seconds(), ts.SubSeconds() );
				st->Set( k + 1, IBPPts );
				break;
		}
	}

}

FSDB::Database FSDB::DatabaseFirebirdFactory( const std::string &serverName, const std::string &databaseName, const std::string &userName, const std::string &password ) { 
	boost::shared_ptr<DatabaseFirebird> p(new DatabaseFirebird(serverName, databaseName, userName, password) );
	return p;
}

void FSDB::DatabaseFirebird::UpdateStatistics( const std::vector<std::string> &tables ) {
	IBPP::Transaction m_tr;
	IBPP::Statement m_st;

	if( !IsTransactionPending() ) {
		m_tr = TransactionFactory( GetIBPPDatabase(), IBPP::amWrite );
		m_tr->Start();
	}
	else {
		m_tr = GetIBPPTransaction();
	}

	try {
		m_st = StatementFactory( GetIBPPDatabase(), m_tr );
		m_st->Prepare( "EXECUTE PROCEDURE SP_SET_STATISTICS_TABLE( ? )" );

		for( size_t k = 0; k < tables.size(); k++ ) {
			m_st->Set( 1, tables[k] );
			m_st->Execute();
		}
	}
	catch( IBPP::SQLException& e ) {
		throw CDataErrorException::IBPPSql2CDataError( e );
	}
	catch( IBPP::LogicException& e ) {
		throw CDataErrorException::IBPPLogic2CDataError( e );
	}
	catch(...) {
		throw;
	}

	if( !IsTransactionPending() ) {
		m_tr->Commit();
	}

}

void FSDB::DatabaseFirebird::Backup( const std::string &backupName ) {

	IBPP::Service svc = IBPP::ServiceFactory( serverName, userName, password );
	svc->Connect();
	svc->StartBackup( databaseName, backupName );
	svc->Wait();
	svc->Disconnect();
}

void FSDB::DatabaseFirebird::Restore( const std::string &backupName, const std::string &databaseName, 
                                      const std::string &_serverName, const std::string &_userName, const std::string &_password ) {

	IBPP::Service svc = IBPP::ServiceFactory( _serverName, _userName, _password );
	svc->Connect();
	svc->StartRestore( backupName, databaseName, 8192 );
	svc->Wait();
	svc->Disconnect();
}

