/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/

/*
Defines a Firebird database, intended as the information needed to connect to it
*/

#ifndef __FSDB_DATABASE_FIREBIRD_H__
#define __FSDB_DATABASE_FIREBIRD_H__

#include "database.h"

#include "../ibpp/ibpp.h"

#include <string.h>

namespace FSDB {

	/// Defines a Firebird database
	class DatabaseFirebird : public DatabaseBase {

	public:

		DatabaseFirebird( const std::string &serverName, const std::string &databaseName, const std::string &userName, const std::string &password );

		void Connect();
		void Disconnect();
        bool IsConnected();

		void TransactionStart();
		void TransactionCommit();
		void TransactionRollback();
		/// returns true if there is an open transaction
		bool IsTransactionPending() { return transactionPending; }

		RecordsetPt CreateRecordset( RecordsetType rt );

		std::string ConvertToSQLString( const FSDB::Variant& var );

		/// return the IBPP database object
		IBPP::Database GetIBPPDatabase() { return db; }
		/// return the transaction object
		IBPP::Transaction GetIBPPTransaction() { return tr; }

		void ExecuteNoReturn( const std::string& sql, Parameters params );
		void ExecuteNoReturn( const std::string& sql );

		int ExecuteReturnInt( const std::string& sql, Parameters params );
		int ExecuteReturnInt( const std::string& sql );

		std::string ExecuteReturnString( const std::string& sql );
		std::string ExecuteReturnString( const std::string& sql, Parameters params );

		/// set parameter values to the statement object
		void SetStatementParameters( IBPP::Statement st, Parameters params );

		/// update the statistics for the tables received in the vector of strings
		void UpdateStatistics( const std::vector<std::string> &tables );

		/// backup the current database
		void Backup( const std::string &backupName );

		/// restore the backup to the given database name
		static void Restore( const std::string &backupName, const std::string &databaseName, 
                             const std::string &serverName, const std::string &userName, const std::string &password );

	protected:

		/// connection info
		std::string serverName, databaseName, userName, password;

		/// the IBPP database object	
		IBPP::Database db;

		/// transaction used by TransactionStart() and similar methods
		IBPP::Transaction tr;

		/// true if there is an open transaction
		bool transactionPending;

		/// Convert an integer to a string.
		/// helper function to convert an integer to a string long at least nChar
		/// if the result string is shorter it is left padded with zeroes
		std::string Int2String( int32_t i, int nChars );

	};

	// returns a newly created DatabaseFirebird object
	Database DatabaseFirebirdFactory( const std::string &serverName, const std::string &databaseName, const std::string &userName, const std::string &password );
}

#endif

