/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/


#include "recordset_fb_rw.h"

#include <assert.h>
#include <stdexcept>

FSDB::Recordset_FB_RW::~Recordset_FB_RW() {
	ReleaseMemory();
}

void FSDB::Recordset_FB_RW::ReleaseMemory() {
	row = NULL;
	for( unsigned int k = 0; k < dataArray.size(); k++ ) {
		row = dataArray[k];
		delete [] dataArray[k];
	}
	dataArray.clear();
}

void FSDB::Recordset_FB_RW::StoreCurrentRowInMemory() {

	// allocates the rowbuffer
	row = new Variant[nColumns];

	// reads the row
	DbInterface->ReadCurrentRow( row );
	// stores it in memory
	dataArray.push_back( row );
	nStoredRows++;
}

void FSDB::Recordset_FB_RW::Open( const std::string &sql, const std::string &tableName, const std::string &primaryKeyName ) {
	// passes an empty parameters collection
	Open( sql, ParametersFactory(), tableName, primaryKeyName );
}

void FSDB::Recordset_FB_RW::Open( const std::string &sql, Parameters params, const std::string &tableName, const std::string &primaryKeyName ) {

	Recordset_FO_RO::Open( sql, params, tableName, primaryKeyName );

	nColumns = DbInterface->GetNColumns();
	nStoredRows = 0;
	currentRowNumber = -1;
    m_readAllRows = false;

	m_bof = DbInterface->IsBOF();
	m_eof = DbInterface->IsEOF();

	if( !IsEmpty() ) {
		StoreCurrentRowInMemory();
		currentRowNumber++;
	}
		
	m_CurrentState = csNormal;
}

void FSDB::Recordset_FB_RW::MoveFirst() {
	
	if( m_bof && m_eof ) return;

    long orgRowNumber = currentRowNumber;
    
    currentRowNumber = 0;
	row = dataArray[currentRowNumber];
	m_bof = false;
	m_eof = false;

    if( m_CurrentState == csDeleted ) {
        RemoveRowFromMemory( orgRowNumber );
        m_CurrentState = csNormal;
    }
}

void FSDB::Recordset_FB_RW::MovePrevious() {
	
	if( m_bof )
		throw std::runtime_error( "MovePrevious() called when at BOF" );

	assert( currentRowNumber >= 0 );

    long orgRowNumber = currentRowNumber;

    currentRowNumber--;
	
	if( m_eof ) m_eof = false;

	if( currentRowNumber < 0 )
		m_bof = true;
	else
		row = dataArray[currentRowNumber];

    if( m_CurrentState == csDeleted ) {
        RemoveRowFromMemory( orgRowNumber );
        m_CurrentState = csNormal;
    }
}

void FSDB::Recordset_FB_RW::MoveNext() {
	
    long orgRowNumber = currentRowNumber;

    if( m_bof ) {
		m_bof = false;
		currentRowNumber = -1;
	}

	if( m_eof ) 
		throw std::runtime_error( "MoveNext() called when at EOF" );

	currentRowNumber++;
	if( currentRowNumber == nStoredRows ) {
		// we have gone past the last stored row: read the next from the database
		if( DbInterface->IsEOF() ) {
			m_eof = true;
		}
		else {
			DbInterface->MoveNext();
            if( DbInterface->IsEOF() ) {
				m_eof = true;
                m_readAllRows = true;
            }
			else
				StoreCurrentRowInMemory();
		}
	}

    if( currentRowNumber >= 0 && currentRowNumber < nStoredRows )
        row = dataArray[currentRowNumber];
    else
        row = NULL;

    if( m_CurrentState == csDeleted ) {
        RemoveRowFromMemory( orgRowNumber );
        m_CurrentState = csNormal;
    }
}



void FSDB::Recordset_FB_RW::MoveLast() {

    long orgRowNumber = currentRowNumber;

    while( !m_eof )
		MoveNext();

	if( !m_bof )
		MovePrevious();

    if( m_CurrentState == csDeleted ) {
        RemoveRowFromMemory( orgRowNumber );
        m_CurrentState = csNormal;
    }
}

void FSDB::Recordset_FB_RW::AddNew() {
	row = new Variant[nColumns];
	Recordset_FO_RW::AddNew();
}


void FSDB::Recordset_FB_RW::Update()
{
	// save this value because it is changed back to csNormal by Recordset_FO_RW::Update()
    CurrentState recordsetState = m_CurrentState;

    // if an error happens during the update an exception is thrown
    // so the following lines are not executed and the row is not added to the recordset
    try {
        Recordset_FO_RW::Update();
    }
    catch(...) {
	    if( recordsetState == csAddNew ) {	
                delete [] row;
                row = NULL;
        }
        throw;
    }

	if( recordsetState == csAddNew ) {	
		// store the new row at the front of the recordset
		dataArray.push_front( row );
		nStoredRows++;
		currentRowNumber++;
	}
}

void FSDB::Recordset_FB_RW::CancelUpdate()
{
	if( m_CurrentState == csAddNew && row != NULL ) {	
            delete [] row;
            row = NULL;
    }

    Recordset_FO_RW::CancelUpdate();
}

void FSDB::Recordset_FB_RW::MoveAbsolute( long newPosition ) {
	if( m_bof && m_eof )
		throw std::runtime_error( "MoveAbsolute() called for an empty recordset" );

	if( newPosition >= nStoredRows )
		throw std::runtime_error( "MoveAbsolute() called for a non-cached row" );

    long orgRowNumber = currentRowNumber;

    currentRowNumber = newPosition;
	row = dataArray[currentRowNumber];
	m_bof = false;
	m_eof = false;

    if( m_CurrentState == csDeleted ) {
        RemoveRowFromMemory( orgRowNumber );
        m_CurrentState = csNormal;
    }
}

long FSDB::Recordset_FB_RW::GetAbsolutePosition() {
	return currentRowNumber;
}

void FSDB::Recordset_FB_RW::SaveAbsolutePosition() {

    if( m_eof && m_bof ) {
        savedRowNumbers.push( spEmpty );
        return;
    }
    
    if( m_eof ) {
        savedRowNumbers.push( spEOF );
    }
    else {
        if( m_bof ) {
            savedRowNumbers.push( spBOF );
        }
        else {
            savedRowNumbers.push( currentRowNumber );
        }
    }
}

void FSDB::Recordset_FB_RW::RestoreAbsolutePosition() {
    if( savedRowNumbers.empty() )
	    throw std::logic_error( "RestoreAbsolutePosition() called without previous SaveAbsolutePosition()" );

    long savedRowNumber = savedRowNumbers.top();
    savedRowNumbers.pop();

    if( savedRowNumber == spEmpty ) {
        return; // recordset was empty, do nothing
    }

    if( savedRowNumber == spEOF ) {
        MoveAbsolute( nStoredRows - 1 );
        MoveNext();
    }
    else {
        if( savedRowNumber == spBOF ) {
            MoveAbsolute( 0 );
            MovePrevious();
        }
        else {
            MoveAbsolute( savedRowNumber );
        }
    }
}

long FSDB::Recordset_FB_RW::GetNumberOfCachedRows() {
	return nStoredRows;
}

bool FSDB::Recordset_FB_RW::HasReadAllRows() {
    if( IsEmpty() )
        return true;
    else
        return m_readAllRows;
}

void FSDB::Recordset_FB_RW::RemoveRowFromMemory( long nRow )
{
	// we should call this function only after we have moved away from the deleted row
    assert( currentRowNumber != nRow );

	delete [] dataArray[nRow];
    dataArray.erase( dataArray.begin() + nRow );
    if( currentRowNumber > nRow )
        currentRowNumber--;
    nStoredRows--;
}



