/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/


#ifndef __FSDB_RECORDSET_FB_RW_H__
#define __FSDB_RECORDSET_FB_RW_H__

#include "recordset_fo_rw.h"

#include <deque>
#include <stack>

namespace FSDB {

    /*  handling of deleted rows.
        if a row is deleted it remains the current row but its content is no more accessible: reading it throws an exception.
        the user needs to move to a different row before reading data. After moving the deleted row disappears.

        this behaviour is implemented in the following way:
        - Delete() deletes the row from the database and sets m_CurrentState = csDeleted.
          the row is not yet removed from dataArray/
        - as soon as the user moves to another row the deleted row is removed from dataArray
    */

	/// A read-write recordset with forward and backward movement.
	/// Data are stored in memory as long as they are read from the database.
	class Recordset_FB_RW : public Recordset_FO_RW {

	public:

		Recordset_FB_RW( RecordsetType rt, DbInterfaceBase *DbInterface ) : 
					Recordset_FO_RW( rt, DbInterface ), m_readAllRows(false) {}
		~Recordset_FB_RW();
		
		/// open a read-write recordset.
		/// receives the sql string, the name of the underlying table and its primary key name.
		void Open( const std::string &sql, const std::string &tableName = "", const std::string &primaryKeyName = "" );
		void Open( const std::string &sql, Parameters params, const std::string &tableName = "", const std::string &primaryKeyName = "" );

		bool IsEOF() { return m_eof; }
		bool IsBOF() { return m_bof; }

		void MoveNext();
		void MovePrevious();
		void MoveFirst();
		void MoveLast();

		void AddNew();

		/// update the db: in case of AddNew() the new row is stored as the first one
		void Update();

		void CancelUpdate();

        void MoveAbsolute( long newPosition );
        long GetAbsolutePosition();
        void SaveAbsolutePosition();
        void RestoreAbsolutePosition();
        long GetNumberOfCachedRows();
        bool HasReadAllRows();

    protected:

        /// simbolic values for the saved absolute position
        enum SavedPositions { 
            spEmpty = -3,       /*!< recordset is empty */
            spEOF = -2,         /*!< recordset is at EOF */
            spBOF = -1          /*!< recordset is at BOF */
        };
        
        /// Data structure that stores the database data
		std::deque<Variant*> dataArray;

		/// number of rows stored in memory
		long nStoredRows;

		/// index of the current row in the container (zero-based)
		long currentRowNumber;

        /// used to temporarily save the current position (zero-based)
        /// possible other values: from the SavedPositions enumeration
        std::stack<long> savedRowNumbers;

		/// end of file flag
		bool m_eof;
		/// BOF flag
		bool m_bof;
        /// flag: true if we have read all the rows in memory
        bool m_readAllRows;

		/// release all the memory used by this instance
		void ReleaseMemory();

		/// store the current recordset row in memory
		void StoreCurrentRowInMemory();

        /// remove a row from memory
        void RemoveRowFromMemory( long nRow );
	};

}

#endif

