/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/


#include "recordset_fo_rw.h"

#include <assert.h>
#include <stdexcept>

FSDB::Recordset_FO_RW::~Recordset_FO_RW() {
	ReleaseMemory();
}

void FSDB::Recordset_FO_RW::ReleaseMemory() {
	if( row != NULL ) {
		delete [] row;
		row = NULL;
	}
}

void FSDB::Recordset_FO_RW::Open( const std::string &sql, const std::string &tableName, const std::string &primaryKeyName ) {
	// passes an empty parameters collection
	Open( sql, ParametersFactory(), tableName, primaryKeyName );
}

void FSDB::Recordset_FO_RW::Open( const std::string &sql, Parameters params, const std::string &tableName, const std::string &primaryKeyName ) {

	Recordset_FO_RO::Open( sql, params, tableName, primaryKeyName );

	nColumns = DbInterface->GetNColumns();

	// allocates the rowbuffer
	row = new Variant[nColumns];

	// reads the first row
	DbInterface->ReadCurrentRow( row );
		
	m_CurrentState = csNormal;
}

void FSDB::Recordset_FO_RW::Close() {

    CheckOpened();
	Recordset_FO_RO::Close();
	ReleaseMemory();
}

void FSDB::Recordset_FO_RW::MoveNext() {
	Recordset_FO_RO::MoveNext();
	DbInterface->ReadCurrentRow( row );
    m_CurrentState = csNormal;
}

FSDB::Variant& FSDB::Recordset_FO_RW::operator []( const std::string& column ) {
    CheckOpened();
    CheckDeleted();
    return row[DbInterface->GetColumnPosition(column)];
}

void FSDB::Recordset_FO_RW::Edit() {
    CheckOpened();
    CheckDeleted();
	if( m_CurrentState != csNormal )
		throw std::runtime_error( "Called Edit() with recordset in Edit or AddNew mode" );
	
	m_CurrentState = csEdit;
}

void FSDB::Recordset_FO_RW::AddNew() {

    CheckOpened();
    CheckDeleted();
	if( m_CurrentState != csNormal )
		throw std::runtime_error( "Called AddNew() with recordset in Edit or AddNew mode" );

	m_CurrentState = csAddNew;

	for( int k = 0; k < nColumns; k ++ ) {
		row[k].SetNull();
		row[k].ClearChanged();
	}
}

void FSDB::Recordset_FO_RW::Update() {
	std::string sql = "";

    CheckOpened();
    CheckDeleted();
	if( m_CurrentState != csEdit && m_CurrentState != csAddNew )
		throw std::runtime_error( "Called Update() without a previous call to Edit() orAddNew()" );

    try {
        if( m_CurrentState == csEdit ) {
            // check if something has changed
            // if nothing has changed do not disturb the database
            bool rowChanged = false;
        	for( int k = 0; k < nColumns; k ++ ) {
                if( row[k].IsChanged() ) {
                    rowChanged = true;
                    break;
                }
            }
            if( rowChanged )
		        DbInterface->UpdateEdit( row );
        }
        if( m_CurrentState == csAddNew ) {
		    DbInterface->UpdateAddNew( row );
        }
    }
    catch(...) {
       	m_CurrentState = csNormal;
        throw;
    }

   	m_CurrentState = csNormal;
}

void FSDB::Recordset_FO_RW::CancelUpdate() {
   	m_CurrentState = csNormal;
}

void FSDB::Recordset_FO_RW::Delete()
{
    CheckOpened();
    CheckDeleted();
    try {
        DbInterface->Delete( row );
    }
    catch(...) {
       	m_CurrentState = csNormal;
        throw;
    }

    m_CurrentState = csDeleted;
}



void FSDB::Recordset_FO_RW::CheckDeleted() {
    if( m_CurrentState == csDeleted )
	    throw std::runtime_error( "The current row has been deleted and it is not accessible any more" );
}

