/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __APP_SETTINGS_H__
#define __APP_SETTINGS_H__

// this class contains the user-definable settings

#include "external_programs.h"

#include <wx/string.h>
#include <wx/docview.h>
#include <vector>

class CAppSettings
{
public:
	CAppSettings(void);
	~CAppSettings(void);

	// possible size of thumbnails, or no thmbnails
	enum ThumbnailSize { tsNoThumbnails, tsSmallThumbnails, tsMediumThumbnails, tsLargeThumbnails };

	// different views handled by the program: fixed locations, removable locations, virtual view
	enum CurrentView { cvFixed, cvRemovable, cvVirtual, cvTimeline, cvSearch };

	// possible monitor profiles
	enum MonitorProfile { mp_sRGB = 0, mp_FileProfile };

	/// copy constructor
	CAppSettings( const CAppSettings& as );

	/// assignment operator
	void operator=(const CAppSettings& as);

    void Copy(const CAppSettings& data);


	// read configuration data
	void Load();
	// save configuration data
	void Save();

	// return the number of column that could be shown in the listview, included the columns that are hidden by the user
	int ListViewColumnsNumber();

	// application settings

	// true if we want to reopen the last used catalog at startup
	bool m_ReopenLastUsedCatalog;
	// minimum duration of a task (in seconds) before playing a beep at task end
	int m_BeepTime;
	// true to force usage of the English version instead of the localized one
	bool m_ForceEnglishLanguage;
	// used to handle the list of recently opened files
	wxFileHistory* m_fileHistory;
	// current thumbnails size
	ThumbnailSize m_CurrentThumbnailViewSize;
	// current view selected by the user
	CurrentView m_CurrentView;
	
	// cataloging options

	// size of the larger side of a thumbnail as stored in the database
	int m_ThumbnailSize;

	// if true, when cataloging a jpeg, tiff or png file, use the embedded thumbnail image to create the catalog thumbnail
	//  (only if there is an embedded thumbnail and if it is large enough)
	// if false always create the thumbnail resizing the full image expanded in memory
	bool m_UseJpegPreview;

	// thumbnails view options
	
	// size of thumbnails in small view
	int m_ThumbnailViewSmallSize;
	// size of thumbnails in medium view
	int m_ThumbnailViewMediumSize;
	// size of thumbnails in large view
	int m_ThumbnailViewLargeSize;

	// color management option

	bool m_EnableColorManagement;
	// monitor profile to use
	MonitorProfile m_MonitorProfile;
	// monitor profile name (if selected by the user)
	wxString m_FileProfileName;

	// database connection settings
	bool dbConnectToServer;
	wxString dbServerName;
	wxString dbUserName;
	wxString dbPassword;
	bool IsLocalhost() { return (dbServerName == wxT("localhost")) || (dbServerName == wxT("127.0.0.1")); }
	// set the correct value in the database access strings
	void GetConnectionData( wxString& serverName, wxString& userName, wxString& password );

	// main frame settings

	/// true if the main frame is maximized
	bool mf_Maximized;
	int mf_XPosition;
	int mf_YPosition;
	int mf_Width;
	int mf_Height;
	int mf_SashPosition;

	// width of the listview columns
	std::vector<int> m_ListviewColWidth;

	// catalog images settings

	// path of the last cataloged volume
	wxString m_CatalogPath;
	// true if we selected removable locations in the dialog
	bool m_CatalogRemovableLocation;
	// true if we selected fixed locations in the dialog
	bool m_CatalogFixedLocation;
	// last fixed location selected by the user
	wxString m_LastFixedLocation;

	// add to virtual folder settings
	
	// list of the last virtual folders used by the "add to virtual folder" command
	// each folder is stored as a wxArrayString of the folders chain, starting from the bottom (root last)
	std::vector<wxArrayString> m_AddVirtualFoldersLastUsed;
	// maximum number of virtual folders that will be saved
	static const int m_MaxAddVirtualFoldersLastUsed = 10;

	// copy files settings
	
	// set to true to extract preview images from raw files
	// set to false to copy the file
	bool m_ExtractPreviewWhenCopyingRawFiles;
	// destination path for the "Copy files" command
	wxString m_CopyFilesDestinationPath;

	// external programs settings

	// external programs
	CExternalPrograms m_ExternalPrograms;
	// where to copy files before executing the external program (if needed)
	wxString m_ExternalProgramsTempPath;

    // timeline view settings

    // start date
    wxDateTime m_TimelineStartDate;
    // end date
    wxDateTime m_TimelineEndDate;
    // threshold
    int m_TimelineThreshold;

private:

	// load an arraystring from the curren config location
	void LoadArrayString( wxConfigBase *pConfig, wxArrayString& as );
	// save an array string to the current location
	void SaveArrayString( wxConfigBase *pConfig, wxArrayString& as );

};


#endif
