/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2009, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    aint with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __CATALOG_IMAGES_FUNCTIONS_H__
#define __CATALOG_IMAGES_FUNCTIONS_H__

#include "fsdb.h"
#include "fs_lcms.h"

#include <wx/string.h>
#include <wx/gauge.h>
#include <wx/filename.h>

#include <map>
#include <vector>

class CCatalogImagesFunctions
{
public:
	CCatalogImagesFunctions( wxWindow *parent, bool *cancelVariable, wxGauge *progressBar = NULL );
	~CCatalogImagesFunctions(void);

	// set data about the removable location to catalog
	void SetRemovableData( wxString volumeName, wxString volumePath, wxString removablePath );

	// set data about the fixed location to catalog
	void SetFixedData( wxString fixedLocationName, wxString fixedLocationPath, wxString fixedPath );

	// catalog the images
	// return an empty string if everything is OK
	// return a formatted error message if there are problems
	wxString Catalog();

	// return a thumbnail of the maximum given size
	// return also the resized image width and height
	static wxImage ResizeImageToThumbnailSize( const wxImage &image, const int thSize, int &thWidth, int &thHeight );

	// returns a buffer containing a jpeg compressed version of the image and the buffer size
	// the buffer is assigned in the function and MUST be deleted by the caller
	static void ConvertImageToBlob( const wxImage &image, char* &buf, int &bufSize );

	// return true if the extension is one of the handled image types
	static bool IsHandledImageType( const wxString &ext );

private:

	// data about each file already cataloged in the db
	// used when updating to know what has changed
	struct CFileData {
		
		CFileData() { IsStillThere = false; }
		
		long FileID;
		wxDateTime DateTime;
		wxULongLong FileSize;
		wxString Sha1;	// hash of the file, to see if it has changed

		bool IsStillThere;	// true if after a volume directory scan the file has been found
	};

	typedef pair <wxString, CFileData> FDPair;

	// return the ID of the volume with the given name
	// add the volume to the DB if it is not present yet
	// volumeType: == "R" for a removable location, "F" for a fixed location
	int GetVolumeID( FSDB::Database db, wxString volumeName, wxString volumeType );

	// return the ID of a folder given its name (not full path)
	// add the folder to the DB if it is not present yet
	int GetSingleFolderID( FSDB::Database db, int volumeID, int fatherID, wxString folderName );

	// return the ID of a folder given its relative path into a volume
	// add the folder to the DB if it is not present yet
	// path is the relative path of the folder into the volume ("" means the root folder)
	// volumeID is the ID of the volume
	int GetFolderID( FSDB::Database db, wxString path, int volumeID );

	// return the ID of the father of a folder, given the folder's relative path into a volume
	// add the father folder to the DB if it is not present yet
	// path is the relative path of the folder into the volume ("" means the root folder)
	// volumeID is the ID of the volume
	// return -1 if path points to the root folder, that has no father
	int GetFolderFatherID( FSDB::Database db, wxString path, int volumeID );

	// catalog the images contained in a folder and recurse in the subfolders
	// update existing images if date or time have changed
	// mainLocation is the path of the main location to catalog. It can be the path to a removable volume or the path of a fixed location
	// secondaryLocation is another location where the same files can be found. It can be empty if it must not be handled
	// mainPath is the path of the folder to be cataloged, relative the main location
	// secondaryPath is the path of the folder to be cataloged, relative to secondary location
	// fatherID is the ID of the folder that contains the folder to be cataloged
	// fatherID == -1 means NULL: we need to catalog the root folder of the volume
	void CatalogSingleFolder( FSDB::Database db, 
		                      wxString mainLocation, 
							  wxString secondaryLocation, 
							  wxString mainPath, 
							  wxString secondaryPath, 
							  int mainVolumeID, 
							  int secondaryVolumeID, 
							  int mainFatherID,
							  int secondaryFatherID);

	// add a trailing path separator to a string containing a path
	wxString AddTrailingPathSeparator( wxString path );

	// store the files already present in the database into a map object
	void ReadFolderFilesFromDB( map<wxString, CFileData> &m, int pathID );

	// store the folders already present in the database into a map object
	void ReadFoldersFromDB( map<wxString, CFileData> &m, int volumeID, int pathID );

	// update the date and size of a file already stored in the database
	void UpdateFileDateSize( int fileID, const wxULongLong &fileSize, const FSDB::Timestamp &fileDateTime );

	// store a row to the FILES table
	// rsFiles must be ready to receive new data: AddNew() or Edit() must have been already called
	//  this function will call rsFiles.Update()
	// rsPathsFiles, rsFiles, rsThumbnail are recorset already opened that will be used to add the row
	// mainPathID is the id of the path (main location) in the PATH table
	// secondaryPathID is the id of the path (secondary location) in the PATH table. it can be == -1 if not used
	// fileID is the primary key of the FILES row that will be written
	void StoreFileRow( FSDB::Database db, 
		FSDB::Recordset rsPathsFiles, 
		FSDB::Recordset rsFiles, 
		FSDB::Recordset rsThumbnails, 
		int mainPathID, 
		int secondaryPathID, 
		wxFileName fnPrimary,
		wxFileName fnSecondary,
		int fileID );

	// add a row to the FILES table
	// rsPathsFiles, rsFiles, rsThumbnail are recorset already opened that will be used to add the row
	// mainPathID is the id of the path (main location) in the PATH table
	// secondaryPathID is the id of the path (secondary location) in the PATH table. it can be == -1 if not used
	void AddFileRow( FSDB::Database db, 
		FSDB::Recordset rsPathsFiles, 
		FSDB::Recordset rsFiles, 
		FSDB::Recordset rsThumbnails, 
		int mainPathID, 
		int secondaryPathID, 
		wxFileName fnPrimary,
		wxFileName fnSecondary );

	// update a row to the FILES table
	// rsPathsFiles, rsThumbnail are recorset already opened that will be used to add the row
	// mainPathID is the id of the path (main location) in the PATH table
	// secondaryPathID is the id of the path (secondary location) in the PATH table. it can be == -1 if not used
	// fileID is the primary key of the FILES row that will be updated
	void UpdateFileRow( FSDB::Database db, 
		FSDB::Recordset rsPathsFiles, 
		FSDB::Recordset rsThumbnails, 
		int mainPathID, 
		int secondaryPathID, 
		wxFileName fnPrimary,
		wxFileName fnSecondary,
		int fileID );

	// store a row in the thumbnails table
	// fileID is the primary key of the image in the FILES row
	// rsThumbnails must be ready to receive new data: AddNew() or Edit() must have been already called
	//  this function will not call Update()
	void StoreThumbnailRow( FSDB::Recordset rsThumbnails, 
		                  wxImage image,
						  int fileID );

	// add a new row to the thumbnails table
	// fileID is the primary key of the image in the FILES row
	void AddThumbnailRow( FSDB::Recordset rsThumbnails, 
		                  wxImage image,
						  int fileID );

	// update an existing thumbnail
	// if the thumbnail is not found append it
	// fileID is the primary key of the image in the FILES row
	void UpdateThumbnailRow( wxImage image,
						  int fileID );

	// report to the user the result of the catalog action
	// return true if the user says "OK", false if the user wants to cancel
	bool ReportCatalogResult();

	// return the SHA1 hash of a file
	wxString ComputeFileSHA1( const wxFileName &fn );

	// window used as a parent for the dialog used to report errors
	wxWindow *parent;

	// true if we want to catalog a removable location
	bool catalogRemovableLocation;
	// true if we want to catalog a fixed location
	bool catalogFixedLocation;

	// name of the removable volume
	wxString volumeName;
	// path of the removable volume
	wxString volumePath;
	// path of the folder to catalog in the removable volume
	wxString removablePath;
	// paths of the folder to catalog, relative to the removable location path
	wxString relativeRemovablePath;

	// name of the fixed location
	wxString fixedLocationName;
	// path of the fixed location
	wxString fixedLocationPath;
	// path of the folder to catalog in the fixed volume
	wxString fixedPath;
	// paths of the folder to catalog, relative to the fixed location path
	wxString relativeFixedPath;

	// if not NULL it is a pointer to a progress bar used to give feedback
	wxGauge *progressBar;

	// reference to a variable in the caller that will be set to true to stop cataloging
	bool *cancelVariable;

	// execution statistics
	long nUpdatedFiles;		// number of files that have been updated
	long nAddedFiles;		// number of files that have been added
	long nDeletedFiles;		// number of files that have been deleted (not counting files in deleted folders)
	long nDeletedFolders;	// number of folders that that have been deleted
	long nUnchangedFiles;	// number of files that have not been changed

	// lists of files couples that caused problems
	// if a file is found in both the fixed and removable location it is assumed to be exactly the same file
	// if the program finds file couples that are different it will store both the file names in one of
	// the following lists.
	// so each list will always contain an even number of file names, since for each file it contains
	// both the name in the removable and fixed location

	// list of files couples with a different SHA1 hash in the removable and fixed location
	vector<wxString> filesListErrorSHA1;

	// list of files couples with a different size and/or datetime in the removable and fixed location
	vector<wxString> filesListErrorSizeDate;

	// list of corrupted images
	vector <wxString> filesListCorruptedImages;

	// to apply color profiles
	CColorProfile m_ColorProfilesHandler;
};


#endif

