/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    aint with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
This class reads the content of two different folders, a primary and a secondary one
It reads all the files in the main folder and its subfolders
It reads all the files in the secondary folder and all its subfolders that are also present in the primary folder
Returns a list of files that are not the same (or are missing) in the two folder trees.

If the secondary path is an empty string it only reads primary path in order to count the files
*/

#ifndef __COMPARE_FOLDERS_H__
#define __COMPARE_FOLDERS_H__

#include <wx/string.h>
#include <wx/datetime.h>
#include <wx/longlong.h>

#include <vector>
#include <map>


class CCompareFolders
{
public:
	// pathPrimary and pathSecondary are the paths that will be compared
	// pathPrimary is also referenced as path1
	// pathSecondary is also referenced as path2
	CCompareFolders( wxString pathPrimary, wxString pathSecondary );

	~CCompareFolders(void);

	// compares the folders
	void Compare();

	// return true if there are differences in the files contained by the two paths
	bool FilesAreDifferent() { return filesOnlyPath1.size() > 0 || filesOnlyPath2.size() > 0 || filesDifferent.size() > 0; }

	// return the list of files only found in path1
	const std::vector<wxString> &GetFilesOnlyPath1() { return filesOnlyPath1; }
	// return the list of files only found in path2
	const std::vector<wxString> &GetFilesOnlyPath2() { return filesOnlyPath2; }
	// return the list of files that are different in path1 and path2
	const std::vector<wxString> &GetFilesDifferent() { return filesDifferent; }

	// return the number of files in the primary path
	int GetFilesNumber() { return (int) files1.size(); }

private:

	// data about each file read from the directories. It will be stored in the map using the file name as a key
	struct CFileData {
		
		CFileData() { FoundInTheOtherFolder = false; }
		
		wxDateTime DateTime;
		wxLongLong FileSize;
		
		// Used only for files in the first folder. While scanning the second folder we check if each file
		// is present in the first folder. If we find it we set this flag to true. Files that remain with the
		// flag set to false are missing from the second folder.
		bool FoundInTheOtherFolder;
	};

	typedef std::pair <wxString, CFileData> FDPair;

	// the paths that will be compared
	wxString pathPrimary, pathSecondary;

	// will contain all the files in path1 and path2
	std::map<wxString, CFileData> files1, files2;

	// list of the files that are only present in path1
	std::vector<wxString> filesOnlyPath1;
	// list of the files that are only present in path2
	std::vector<wxString> filesOnlyPath2;
	// list of the files that are different
	std::vector<wxString> filesDifferent;

	// read all the files in path (included subfolders) and store them in the map object
	// basePath is the starting path and it will not be stored because it changes between the two paths that will be checked
	// the map opject will store the concatenation of (path + file name) that will be comparable
	// if recurse == true it will also scan subfolders
	void ReadAllFiles( wxString basePath, wxString path, std::map<wxString, CFileData> &files, bool recurse );

#ifdef __WXMSW__
	// Windows-specific version because the standard one was rather slow
	void ReadAllFilesWindows( wxString basePath, wxString path, std::map<wxString, CFileData> &files, bool recurse );
#endif

	// read the files in the secondary folder.
	// It works in a way similar to ReadAllFiles(), but it only analyzes the subfolders (1st level) that
	// are contained in the primary path.
	// We need to work in this way because the fixed path might contain many more folders than the removable one:
	// those folders might be stored in other removable volumes
	void ReadAllFilesInSecondaryFolder( wxString basePath, wxString path, wxString primaryPath, std::map<wxString, CFileData> &files );
};


#endif
