/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "filedata_array.h"
#include "utils.h"

#include <wx/mstream.h>

// function used to sort the array
bool filedataSortFunction( CFileData* fd1, CFileData* fd2 )
{
	return *fd1 < *fd2;
}

bool CFileData::operator <( const CFileData &other )
{
	if( imageDateTime == other.imageDateTime )
		return name.Upper() < other.name.Upper();
	else
		return imageDateTime < other.imageDateTime;
}

void CFileData::DeleteThumbnail()
{
	_hasThumbnail = false;
}

void CFileData::SetThumbnail( wxImage tmbImage )
{
	wxASSERT( tmbImage.IsOk() );
	
	thumbnail = tmbImage;
	_hasThumbnail = true;
}

CFileDataArray::CFileDataArray(void)
{
}

CFileDataArray::~CFileDataArray(void)
{
	Clear();
}


void CFileDataArray::Clear() {

	for( unsigned int k = 0; k < dataArray.size(); k++ ) {
		delete dataArray[k];
	}
	dataArray.clear();
}

void CFileDataArray::FillFromRecordset( FSDB::Database db, FSDB::Recordset rs, bool isVirtual, bool append ) {

	// clear the list
	if( !append ) Clear();

	while( !rs.IsEOF() ) {
		CFileData* fd = new CFileData;
		fd->ID = rs["FILE_ID"];
		if( isVirtual ) {
			fd->virtualID = rs["VIRTUAL_FILE_ID"];
		}
		else {
			fd->virtualID = -1;
		}
		fd->name = CUtils::DBstd2wx( rs["FILE_NAME"] );
		fd->size = CUtils::i64_fsdb2wx( rs["FILE_SIZE"].GetAsLargeInt() );
		if( !rs["WIDTH"].IsNull() )
			fd->width = rs["WIDTH"];
		else
			fd->width = -1;
		if( !rs["HEIGHT"].IsNull() )
			fd->height = rs["HEIGHT"];
		else
			fd->height = -1;
		fd->ext = CUtils::DBstd2wx( rs["FILE_EXT"] );
		fd->description = CUtils::DBstd2wx( rs["FILE_DESCRIPTION"] );
		fd->fileDateTime = CUtils::dt_fsdb2wx( rs["FILE_DATETIME"] );
		if( !rs["IMAGE_DATETIME"].IsNull() ) 
			fd->imageDateTime = CUtils::dt_fsdb2wx( rs["IMAGE_DATETIME"] );
		else
			fd->imageDateTime = wxInvalidDateTime;
		fd->rotation = (CUtils::ImageRotation) ((int) rs["ROTATION"]);
		if( !rs["THUMBNAIL"].IsNull() ) {
			// load thumbnail data
			const void *buf;
			int size;
			rs["THUMBNAIL"].GetAsBlobConstPointer( buf, size );
			wxMemoryInputStream mis( buf, size );
			fd->thumbnail.LoadFile( mis, wxBITMAP_TYPE_JPEG );

			fd->thWidth = rs["TH_WIDTH"];
			fd->thHeight = rs["TH_HEIGHT"];

			fd->_hasThumbnail = true;
		}
		else {
			fd->_hasThumbnail = false;
		}
		fd->metadata.LoadFromDB( fd->ID, db );

		dataArray.push_back( fd );

		rs.MoveNext();
	}
}

int CFileDataArray::FillFromRecordsetPartial( FSDB::Database db, FSDB::Recordset rs, bool isVirtual, int maxRows )
{
	int readRows = 0;	// number of rows read from the recordset

	db->TransactionStart();
	while( !rs.IsEOF() && readRows < maxRows ) {
		CFileData* fd = new CFileData;
		fd->ID = rs["FILE_ID"];
		if( isVirtual ) {
			fd->virtualID = rs["VIRTUAL_FILE_ID"];
		}
		else {
			fd->virtualID = -1;
		}
		fd->name = CUtils::DBstd2wx( rs["FILE_NAME"] );
		fd->size = CUtils::i64_fsdb2wx( rs["FILE_SIZE"].GetAsLargeInt() );
		if( !rs["WIDTH"].IsNull() )
			fd->width = rs["WIDTH"];
		else
			fd->width = -1;
		if( !rs["HEIGHT"].IsNull() )
			fd->height = rs["HEIGHT"];
		else
			fd->height = -1;
		fd->ext = CUtils::DBstd2wx( rs["FILE_EXT"] );
		fd->description = CUtils::DBstd2wx( rs["FILE_DESCRIPTION"] );
		fd->fileDateTime = CUtils::dt_fsdb2wx( rs["FILE_DATETIME"] );
		if( !rs["IMAGE_DATETIME"].IsNull() ) 
			fd->imageDateTime = CUtils::dt_fsdb2wx( rs["IMAGE_DATETIME"] );
		else
			fd->imageDateTime = wxInvalidDateTime;
		fd->rotation = (CUtils::ImageRotation) ((int) rs["ROTATION"]);
		if( !rs["THUMBNAIL"].IsNull() ) {
			// load thumbnail data
			const void *buf;
			int size;
			rs["THUMBNAIL"].GetAsBlobConstPointer( buf, size );
			wxMemoryInputStream mis( buf, size );
			fd->thumbnail.LoadFile( mis, wxBITMAP_TYPE_JPEG );

			fd->thWidth = rs["TH_WIDTH"];
			fd->thHeight = rs["TH_HEIGHT"];

			fd->_hasThumbnail = true;
		}
		else {
			fd->_hasThumbnail = false;
		}
		fd->metadata.LoadFromDB( fd->ID, db );

		dataArray.push_back( fd );

		rs.MoveNext();
		readRows++;
	}
	db->TransactionCommit();

	return readRows;
}


void CFileDataArray::Sort()
{
	std::sort( dataArray.begin(), dataArray.end(), filedataSortFunction );
}



