/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "fixed_locations.h"
#include "utils.h"

#include "wx/config.h"

CFixedLocations::CFixedLocations(void)
{
}

CFixedLocations::~CFixedLocations(void)
{
	Clear();
}

void CFixedLocations::Clear() {

	for( unsigned int k = 0; k < dataArray.size(); k++ ) {
		delete dataArray[k];
	}
	dataArray.clear();
}

CFixedLocations::Error CFixedLocations::Append( wxString name, wxString path, wxString description ) {

	CFixedLocations::Error err = CheckData( name, path, description, -1 );
	if( err != eNoError )
		return err;

	CSingleFixedLocation *sl = new CSingleFixedLocation;
	sl->name = name;
	sl->path = path;
	sl->description = description;
	dataArray.push_back( sl );

	return eNoError;
}

CFixedLocations::Error CFixedLocations::Edit( wxString name, wxString path, wxString description, int index ) {

	if( index < 0 || index >= (int) dataArray.size() )
		throw std::runtime_error( "Invalid index" );

	CFixedLocations::Error err = CheckData( name, path, description, index );
	if( err != eNoError )
		return err;

	CSingleFixedLocation *sl = dataArray[index];
	sl->name = name;
	sl->path = path;
	sl->description = description;

	return eNoError;
}

CFixedLocations::Error CFixedLocations::CheckData( wxString name, wxString path, wxString description, int index ) {

	if( name == wxEmptyString )
		return eMissingName;
	if( path == wxEmptyString )
		return eMissingPath;

	for( unsigned int k = 0; k < dataArray.size(); k++ ) {
		if( k != index ) {

			if( dataArray[k]->name == name )
				return eDuplicateName;

			if( dataArray[k]->path == path )
				return eDuplicatePath;

			if( dataArray[k]->path.Len() < path.Len() ) 
				if( path.StartsWith(dataArray[k]->path) )
					return ePathContained;

			if( path.Len() < dataArray[k]->path.Len() ) 
				if( dataArray[k]->path.StartsWith(path) )
					return ePathContains;

		}
	}

	return eNoError;
}

void CFixedLocations::Remove( int index ) {

	if( index < 0 || index >= (int) dataArray.size() )
		throw std::runtime_error( "Invalid index" );

	delete dataArray[index];
	dataArray.erase( dataArray.begin() + index );
}

void CFixedLocations::Load() {

	if( FixedLocationUUID == wxEmptyString )
		throw std::runtime_error( "Missing UUID for fixed locations" );

	Clear();
	
	wxConfigBase *pConfig = wxConfigBase::Get();

	pConfig->SetPath( wxT("/FixedLocations/") + FixedLocationUUID );
	int nLocations = pConfig->Read( wxT("nLocations"), (long) 0 );

	for( int k = 0; k < nLocations; k++ ) {
		wxString name = pConfig->Read( wxT("Name") + CUtils::long2string(k), wxEmptyString );
		wxString path = pConfig->Read( wxT("Path") + CUtils::long2string(k), wxEmptyString );
		wxString description = pConfig->Read( wxT("Description") + CUtils::long2string(k), wxEmptyString );

		if( name != wxEmptyString && path != wxEmptyString )
			Append( name, path, description );
	}
}

void CFixedLocations::Save() {

	if( FixedLocationUUID == wxEmptyString )
		throw std::runtime_error( "Missing UUID for fixed locations" );

	wxConfigBase *pConfig = wxConfigBase::Get();

	pConfig->SetPath( wxT("/FixedLocations/") + FixedLocationUUID );
	pConfig->Write( wxT("nLocations"), (long) dataArray.size() );

	for( unsigned int k = 0; k < dataArray.size(); k++ ) {
		pConfig->Write( wxT("Name") + CUtils::long2string(k), dataArray[k]->name );
		pConfig->Write( wxT("Path") + CUtils::long2string(k), dataArray[k]->path );
		pConfig->Write( wxT("Description") + CUtils::long2string(k), dataArray[k]->description );

	}
}

void CFixedLocations::SetUUID( wxString sUUID ) {

	if( sUUID == wxEmptyString )
		throw std::runtime_error( "Empty UUID parameter" );
	
	FixedLocationUUID = sUUID;
	Load();
}

wxString CFixedLocations::GetPathByName( wxString locationName )
{
	wxString retVal = wxEmptyString;
	for( size_t k = 0; k < dataArray.size(); k++ ) {
		if( dataArray[k]->GetName() == locationName ) {
			retVal = dataArray[k]->GetPath();
			break;
		}
	}
	return retVal;
}

