/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#include "fs_image.h"
#include "utils.h"

#include <wx/thread.h>

IMPLEMENT_DYNAMIC_CLASS(fsImage, wxImage)

// to easily swithc between the float and doubler datatype
typedef double FLOAT_NUM;

typedef unsigned char unsigned8;

// numero di bit per cui sono shiftati a sinistra i pesi per usare una rappresentazione intera
// bits are shifted left by this number using an integer to represent a floating number
#define SHIFT_WEIGHTS 11
// twice as before
#define SHIFT_WEIGHTS2 22


/////////////////////////////////////////////////////////////////
// this a function that rounds a float to an integer
// it should be much faster than a simple cast
/////////////////////////////////////////////////////////////////

static inline long FSRound( FLOAT_NUM a ) {
#ifndef __WXGTK__
	long retVal;

	__asm fld a
	__asm fistp retVal

	return retVal;
 #else
	// I don't know how to do it under Linux
	return (long) (a > 0.0 ? (a + 0.5) : (a - 0.5));
 #endif
}


/////////////////////////////////////////////////////////////////
// functions used for the interpolator kernel
/////////////////////////////////////////////////////////////////

// the Sinc() function
static FLOAT_NUM Sinc( FLOAT_NUM x ) {
	const double PI = 3.14159265358979;

	if( x == 0.0f )
		return 1.0f;
	else
		return sin(PI*x)/(PI*x);
}

// compute the Lanczos function with the specifie number of lobes
static FLOAT_NUM Lanczos(int nLobes, FLOAT_NUM x)
{
	if( fabs(x) >= nLobes )	return 0;

	return Sinc(x) * Sinc(x/nLobes);
}

// compute the value of a blackman windowed sinc with the specifie number of lobes
static FLOAT_NUM Blackman(int nLobes, FLOAT_NUM x)
{
	const double PI = 3.14159265358979;
	FLOAT_NUM w;

	if( fabs(x) >= nLobes )	return 0;

	w = 0.42 + 0.50*cos(PI*x/nLobes) + 0.08*cos(2.*PI*x/nLobes);
	return Sinc(x)*w;
}


// compute the value of a windowed sinc with the specifie number of lobes
static FLOAT_NUM BlackmanHarris(int nLobes, FLOAT_NUM x)
{
	const double PI = 3.14159265358979;
	FLOAT_NUM w;

	if( fabs(x) >= nLobes )	return 0;

	w = 0.42323 + 0.49755*cos(PI*x/nLobes) + 0.07922*cos(2.*PI*x/nLobes);
	return Sinc(x)*w;
}

// compute the weights used for the interpolation 
// will be called once for the weights along the x axis and one along the y axis
// nPixels: number of pixels in the row or column of the output image
// weights: array that will contain the weights
// ratio: enlargement (<1) or reduction (>1) ratio
// nPoints: number of points on each side
static void ComputeWeights( int nPixels, int *weights, FLOAT_NUM ratio, int nPoints )
{
	int k, i;
	FLOAT_NUM x, delta, tot, corr;
	FLOAT_NUM *tmpWeights;
	int *pt;
	int shiftFactor;	// multiplicative factor corresponding to a SHIFT_WEIGHTS shift to the left

	tmpWeights = new FLOAT_NUM[nPoints*2 + 10];
	shiftFactor = FSRound(pow(2, SHIFT_WEIGHTS));
	pt = weights;
	for( k = 0; k < nPixels; k++ ) {
		x = k*ratio;	// theoretical position
		
		// position of the first point to the left to be considered in the filter function graph
		x = floor(x) - x - (nPoints - 1);
		delta = 1;
		
		// if the image is reduced
		if( ratio > 1 ) {
			x = x / ratio;
			delta = delta / ratio;
		}

		// compute the weights for this pixel
		tot = 0;
		for( i = 0; i < nPoints*2; i++ ) {
			tmpWeights[i] = Lanczos( nPoints, x );
//			tmpWeights[i] = Blackman( nPoints, x );
//			tmpWeights[i] = BlackmanHarris( nPoints, x );
			tot += tmpWeights[i];
			x += delta;
		}

		// normalize weights
		corr = 1/tot;	// correction coefficient to make the weights sum == 1
		for( i = 0; i < nPoints*2; i++ )
			*pt++ = FSRound(tmpWeights[i]*corr*shiftFactor);
	}

	delete tmpWeights;
}

/////////////////////////////////////////////////////////////////
// static functions, needed to run on different threads
/////////////////////////////////////////////////////////////////

// used to scale an image with a good interpolator
// the parameters are the pointer to the image RGB and alpha buffers, and the images' sizes
// firstRow and lastRow define the output rows that must be created by a function call
static void fsImageScaleFS( unsigned8* src_data, unsigned8* src_alpha, int xSizeOrg, int ySizeOrg, 
						    unsigned8* dst_data, unsigned8* dst_alpha, int xSizeNew, int ySizeNew,
							int firstRow, int lastRow )
{
	// declare some variables

	int k;
	
	// support of the kernel: e.g. Lanczos2 has support == 2
	int support = 2;
	// original width / new width along the x axis
	FLOAT_NUM xRatio = FLOAT_NUM(xSizeOrg) / xSizeNew;
	// original height / new height along the t axis
	FLOAT_NUM yRatio = FLOAT_NUM(ySizeOrg) / ySizeNew;
	// rows (x axis): number of points used to interpolate on each side
	// the total number of points is nPointsX*2
	int nPointsX;
	// columns (y axis): number of points used to interpolate on each side
	// the total number of points is nPointsX*2
	int nPointsY;

	if( xRatio <= 1 )
		nPointsX = support;	// enlarge
	else
		nPointsX = int(support*xRatio);	// reduce
	if( yRatio <= 1 )
		nPointsY = support;
	else
		nPointsY = int(support*yRatio);

	// to speedup computations
	int nPointsX2 = nPointsX * 2;
	int nPointsY2 = nPointsY * 2;

	// used to store temporary values during interpolation
	// uses 4 elements: RGBA
	int *vPlanes[4];
	for( k = 0; k < 4; k++ )
		vPlanes[k] = new int[nPointsX*2 + 10];

	// weights allocation
	// the weights are stored sequentially for each pixel
	// first the nPointsX*2 weights of the first pixel, then those of the second pixel ad so on
	int *xWeights = new int[nPointsX*2*xSizeNew];
	int *yWeights = new int[nPointsY*2*ySizeNew];

	// allocate memory for the pointers to the rows used for the y axis interpolation
	unsigned8 **ptRighe = new unsigned8*[nPointsY2];			// for RGB data
	unsigned8 **ptRigheAlpha = new unsigned8*[nPointsY2];	// for alpha channel

	// compute weights:
	// x axis
	ComputeWeights( xSizeNew, xWeights, xRatio, nPointsX );
	// y axis
	ComputeWeights( ySizeNew, yWeights, yRatio, nPointsY );

	// allocate and setup clampTable8
	// used to quickly force values to be between 0 and 255
	unsigned8 *clampTable8Base = new unsigned8[1000];
	unsigned8 *clampTable8 = clampTable8Base + 350;
	for( k = -348; k < 0; k++ ) clampTable8[k] = 0;
	for( k = 0; k <= 255; k++ ) clampTable8[k] = k;
	for( k = 256; k < 640; k++ ) clampTable8[k] = 255;

	dst_data += firstRow*xSizeNew*3;	// move to the first output pixel
	dst_alpha += firstRow*xSizeNew;
	for( int dsty = firstRow; dsty <= lastRow; dsty++ ) {
		// loop over the output lines
		int k, i, nRow;
		FLOAT_NUM x, y;
		y = dsty*yRatio;

		// store the row pointers
		// ptRighe contains the address of the start of the rows in the original image that will be used for interpolation
		// used to save bound checks when interpolating each pixel
		//	k = FSRound(y - 0.5f) - (nPointsY - 1);
		k = FSRound(floor(y) - (nPointsY - 1));	// index of the first row that we need
		for( i = 0; i < nPointsY2; k++, i++ ) {
			// check that the row is not outside the image
			nRow = k;
			if( nRow < 0 ) nRow = 0;
			if( nRow >= ySizeOrg ) nRow = ySizeOrg - 1;
			ptRighe[i] = src_data + nRow*xSizeOrg*3;
			ptRigheAlpha[i] = src_alpha + nRow*xSizeOrg;
		}
		
		// now interpolate the pixels of the current output row
		for( int dstx = 0; dstx < xSizeNew; dstx++ ) {
			int *w;			// interpolation weights
			int tmpPlanes[4];	// used to temporarily store interpolation data
			int jtl;		// (top left)
			int ki;

			x = dstx*xRatio;
			// coordinate of the top left pixel in the source image
		//	jtl = FSRound(x - 0.5f) - (nPointsX - 1);
			jtl = FSRound(floor(x) - (nPointsX - 1));

			// set weights for interpolation along the x axis
			w = xWeights + (dstx*nPointsX2);

			// decide if interpolating this pixel will lead us outside the source image
			bool needClamp = false;
			if( jtl < 0 ) needClamp = true;
			if( jtl + nPointsX2 > xSizeOrg - 1 ) needClamp = true;

			// interpolate the various lines along the x axis
			for( ki = 0; ki < nPointsY2; ki++ ) {
				int *dpt = tmpPlanes;
				*dpt++ = 0;
				*dpt++ = 0;
				*dpt++ = 0;
				*dpt = 0;

				// separate clamp/noclamp to optimize
				if( needClamp ) {
					for( int kj = 0; kj < nPointsX2; kj++ ) {
						int j = kj + jtl;
						if( j < 0 ) j = 0;
						if( j >= xSizeOrg ) j = xSizeOrg - 1;
						// now j correspond to the pixel that we need to consider
						unsigned8 *cp = ptRighe[ki] + j*3;
						tmpPlanes[0] += (*cp++)*w[kj];
						tmpPlanes[1] += (*cp++)*w[kj];
						tmpPlanes[2] += (*cp)*w[kj];
						if( src_alpha ) {
							cp = ptRigheAlpha[ki] + j;
							tmpPlanes[3] += (*cp)*w[kj];
						}
					}
				}
				else {
					unsigned8 *cp = ptRighe[ki] + jtl*3;
					for( int kj = 0; kj < nPointsX2; kj++ ) {
						// compute along one row
						dpt = tmpPlanes;
						*dpt += (*cp++)*w[kj];
						dpt++;
						*dpt += (*cp++)*w[kj];
						dpt++;
						*dpt += (*cp++)*w[kj];
						if( src_alpha ) {
							dpt++;
							cp = ptRigheAlpha[ki] + jtl;
							*dpt += (*cp++)*w[kj];
						}
					}
				}

				// store the result for the current line
				vPlanes[0][ki] = tmpPlanes[0];
				vPlanes[1][ki] = tmpPlanes[1];
				vPlanes[2][ki] = tmpPlanes[2];
				if( src_alpha ) 
					vPlanes[3][ki] = tmpPlanes[3];
			}

			// set weights for interpolation along the y axis
			w = yWeights + (dsty*nPointsY2);

			// now the final interpolation along the y axis
			tmpPlanes[0] = tmpPlanes[1] = tmpPlanes[2] = tmpPlanes[3] = 0;
			for( ki = 0; ki < nPointsY2; ki++ ) {
				tmpPlanes[0] += vPlanes[0][ki]*w[ki];
				tmpPlanes[1] += vPlanes[1][ki]*w[ki];
				tmpPlanes[2] += vPlanes[2][ki]*w[ki];
				if( src_alpha ) tmpPlanes[3] += vPlanes[3][ki]*w[ki];
			}

			// rescale the found values
			tmpPlanes[0] >>= SHIFT_WEIGHTS2;
			tmpPlanes[1] >>= SHIFT_WEIGHTS2;
			tmpPlanes[2] >>= SHIFT_WEIGHTS2;
			if( src_alpha ) tmpPlanes[3] >>= SHIFT_WEIGHTS2;

			// check that values stay in the 0-255 range
			*dst_data++ = clampTable8[tmpPlanes[0]];
			*dst_data++ = clampTable8[tmpPlanes[1]];
			*dst_data++ = clampTable8[tmpPlanes[2]];
			if( src_alpha ) 
				*dst_alpha++ = clampTable8[tmpPlanes[3]];
		}
	}

	delete clampTable8Base;

	delete ptRigheAlpha;
	delete ptRighe;

	delete yWeights;
	delete xWeights;

	for( k = 0; k < 4; k++ )
		delete vPlanes[k];

}


// used to scale image data by half
// the parameters are the pointer to the image RGB and alpha buffers, and the source image size
// firstRow and lastRow define the output rows that must be created by a function call
static void fsImageScaleHalf( unsigned8* src_data, unsigned8* src_alpha, int srcWidth, int srcHeight, unsigned8* dst_data, unsigned8* dst_alpha,
							  int firstRow, int lastRow )
{
	int width = srcWidth / 2;
	int height = srcHeight / 2;

	// this will point to the destination image
	unsigned8 *ptOut = dst_data + firstRow*width*3;

	for( int dsty = firstRow; dsty <= lastRow; dsty++ ) {

		// these will point to the used rows in the source image
		unsigned8 *ptIn1 = src_data + dsty*2*srcWidth*3;
		unsigned8 *ptIn2 = ptIn1 + srcWidth*3;

		for( int dstx = 0; dstx < width; dstx++ ) {
			int tmp1, tmp2, tmp3;	// RGB components
			tmp1 = *ptIn1++;
			tmp2 = *ptIn1++;
			tmp3 = *ptIn1++;
			tmp1 += *ptIn1++;
			tmp2 += *ptIn1++;
			tmp3 += *ptIn1++;
			tmp1 += *ptIn2++;
			tmp2 += *ptIn2++;
			tmp3 += *ptIn2++;
			tmp1 += *ptIn2++;
			tmp2 += *ptIn2++;
			tmp3 += *ptIn2++;

			*ptOut++ = tmp1/4;
			*ptOut++ = tmp2/4;
			*ptOut++ = tmp3/4;
		}
	}

    if ( src_alpha ) {
		// resize the alpha channel
		unsigned8 *ptOut = dst_alpha;
		for( int dsty = 0; dsty < height; dsty++ ) {
			unsigned8 *ptIn1 = src_alpha + dsty*srcWidth;
			unsigned8 *ptIn2 = ptIn1 + srcWidth*2;
			for( int dstx = 0; dstx < width; dstx++ ) {
				int tmp1;
				tmp1 = *ptIn1++;
				tmp1 += *ptIn1++;
				tmp1 += *ptIn2++;
				tmp1 += *ptIn2++;
				*ptOut++ = tmp1/4;
			}
		}
	}
}

/////////////////////////////////////////////////////////////////
// thread used for ScaleFS()
/////////////////////////////////////////////////////////////////
class ScaleFSThread : public wxThread
{
public:

	ScaleFSThread( unsigned8* src_data, unsigned8* src_alpha, int xSizeOrg, int ySizeOrg, unsigned8* dst_data, unsigned8* dst_alpha,
							  int xSizeNew, int ySizeNew, int firstRow, int lastRow );
	virtual void *Entry();

private:

	unsigned8* src_data;
	unsigned8* src_alpha;
	int xSizeOrg;
	int ySizeOrg;
	unsigned8* dst_data;
	unsigned8* dst_alpha;
	int xSizeNew;
	int ySizeNew;
	int firstRow;
	int lastRow;
};

ScaleFSThread::ScaleFSThread( unsigned8* src_data, unsigned8* src_alpha, int xSizeOrg, int ySizeOrg, unsigned8* dst_data, unsigned8* dst_alpha,
								 int xSizeNew, int ySizeNew, int firstRow, int lastRow ) : wxThread( wxTHREAD_JOINABLE )
{
	this->src_data = src_data;
	this->src_alpha = src_alpha;
	this->xSizeOrg = xSizeOrg;
	this->ySizeOrg = ySizeOrg;
	this->dst_data = dst_data;
	this->dst_alpha = dst_alpha;
	this->xSizeNew = xSizeNew;
	this->ySizeNew = ySizeNew;
	this->firstRow = firstRow;
	this->lastRow = lastRow;
}

void *ScaleFSThread::Entry()
{
	fsImageScaleFS( src_data, src_alpha, xSizeOrg, ySizeOrg, dst_data, dst_alpha, xSizeNew, ySizeNew, firstRow, lastRow );
	return NULL;
}



/////////////////////////////////////////////////////////////////
// thread used for ScaleHalf()
/////////////////////////////////////////////////////////////////
class ScaleHalfThread : public wxThread
{
public:

	ScaleHalfThread( unsigned8* src_data, unsigned8* src_alpha, int srcWidth, int srcHeight, unsigned8* dst_data, unsigned8* dst_alpha,
							  int firstRow, int lastRow );
	virtual void *Entry();

private:

	unsigned8* src_data;
	unsigned8* src_alpha;
	int srcWidth;
	int srcHeight;
	unsigned8* dst_data;
	unsigned8* dst_alpha;
	int firstRow;
	int lastRow;
};

ScaleHalfThread::ScaleHalfThread( unsigned8* src_data, unsigned8* src_alpha, int srcWidth, int srcHeight, unsigned8* dst_data, unsigned8* dst_alpha,
								 int firstRow, int lastRow ) : wxThread( wxTHREAD_JOINABLE )
{
	this->src_data = src_data;
	this->src_alpha = src_alpha;
	this->srcWidth = srcWidth;
	this->srcHeight = srcHeight;
	this->dst_data = dst_data;
	this->dst_alpha = dst_alpha;
	this->firstRow = firstRow;
	this->lastRow = lastRow;
}

void *ScaleHalfThread::Entry()
{
	fsImageScaleHalf( src_data, src_alpha, srcWidth, srcHeight, dst_data, dst_alpha, firstRow, lastRow );
	return NULL;
}


/////////////////////////////////////////////////////////////////
// fsImage class
/////////////////////////////////////////////////////////////////

fsImage::fsImage( int width, int height, bool clear )
{
	Init();
    Create( width, height, clear );
}

fsImage::fsImage( int width, int height, unsigned char* data, bool static_data )
{
	Init();
    Create( width, height, data, static_data );
}

fsImage::fsImage( int width, int height, unsigned char* data, unsigned char* alpha, bool static_data )
{
	Init();
    Create( width, height, data, alpha, static_data );
}

fsImage::fsImage( const wxString& name, long type, int index )
{
	Init();
    LoadFile( name, type, index );
}

fsImage::fsImage( const wxString& name, const wxString& mimetype, int index )
{
	Init();
    LoadFile( name, mimetype, index );
}

#if wxUSE_STREAMS
fsImage::fsImage( wxInputStream& stream, long type, int index )
{
	Init();
    LoadFile( stream, type, index );
}

fsImage::fsImage( wxInputStream& stream, const wxString& mimetype, int index )
{
	Init();
    LoadFile( stream, mimetype, index );
}
#endif // wxUSE_STREAMS

fsImage::fsImage(const char* const* xpmData)
{
	Init();
    Create(xpmData);
}

fsImage::~fsImage()
{
	// wxImage is reference counted so the destructor might be called many times with the same profile address
	// make sure that we delete memory only when reference count is 1
	if( this->IsOk() ) {
		if( this->GetRefData()->GetRefCount() == 1 ) {
			DeleteICCProfile();
		}
	}
}

void fsImage::Init()
{
	m_ICCProfile = NULL;
	m_ICCProfileSize = 0;
}



fsImage fsImage::ScaleFS( int width, int height ) const
{
	// first, if we are downsizing a lot use ScaleHalf to gain speed
	fsImage input_image = *this;
	while( width*2 < input_image.GetWidth() && height*2 < input_image.GetHeight() ) {
		input_image = input_image.ScaleHalf();
	}

	// declare some variables
	
	// new width of the image
	int xSizeNew = width;
	// new height of the image
	int ySizeNew = height;
	// original width of the image
	int xSizeOrg = input_image.GetWidth();
	// original height of the image
	int ySizeOrg = input_image.GetHeight();

	// create the empty output image
	fsImage ret_image;
    ret_image.Create(width, height, false);
    unsigned8* src_data = input_image.GetData();
    unsigned8* src_alpha = input_image.GetAlpha();
    unsigned8* dst_data = ret_image.GetData();	// will advance to point to the bytes to be written for the current pixel
    unsigned8* dst_alpha = NULL;
    if ( src_alpha ) {
        ret_image.SetAlpha();
        dst_alpha = ret_image.GetAlpha();
    }

	int half = height/2;
	ScaleFSThread thread1( src_data, src_alpha, xSizeOrg, ySizeOrg, dst_data, dst_alpha, xSizeNew, ySizeNew, 0, half - 1 );
	if( thread1.Create() != wxTHREAD_NO_ERROR ) {
		CUtils::MsgErr( _("Unable to create thread") );
	}
	ScaleFSThread thread2( src_data, src_alpha, xSizeOrg, ySizeOrg, dst_data, dst_alpha, xSizeNew, ySizeNew, half, height - 1 );
	if( thread2.Create() != wxTHREAD_NO_ERROR ) {
		CUtils::MsgErr( _("Unable to create thread") );
	}
	thread1.Run();
	thread2.Run();
	thread1.Wait();
	thread2.Wait();

//	fsImageScaleFS( src_data, src_alpha, xSizeOrg, ySizeOrg, dst_data, dst_alpha, xSizeNew, ySizeNew, 0, ySizeNew - 1 );

	return ret_image;
}

fsImage fsImage::ScaleHalf()
{
	// this function simply computes each destination pixel as the mean value of 4 pixels in the source image

	// compute the new size
	// we will throw away one pixel if the size is an odd number
	int srcWidth = GetWidth();
	int srcHeight = GetHeight();
	int width = srcWidth / 2;
	int height = srcHeight / 2;

	// create the empty output image
	fsImage ret_image;
    ret_image.Create(width, height, false);
    unsigned8* src_data = GetData();
    unsigned8* src_alpha = GetAlpha();
    unsigned8* dst_data = ret_image.GetData();
    unsigned8* dst_alpha = NULL;
    if ( src_alpha ) {
        ret_image.SetAlpha();
        dst_alpha = ret_image.GetAlpha();
    }

	int half = height/2;
	ScaleHalfThread thread1( src_data, src_alpha, srcWidth, srcHeight, dst_data, dst_alpha, 0, half - 1 );
	if( thread1.Create() != wxTHREAD_NO_ERROR ) {
		CUtils::MsgErr( _("Unable to create thread") );
	}
	ScaleHalfThread thread2( src_data, src_alpha, srcWidth, srcHeight, dst_data, dst_alpha, half, height - 1 );
	if( thread2.Create() != wxTHREAD_NO_ERROR ) {
		CUtils::MsgErr( _("Unable to create thread") );
	}
	thread1.Run();
	thread2.Run();
	thread1.Wait();
	thread2.Wait();

//	fsImageScaleHalf( src_data, src_alpha, srcWidth, srcHeight, dst_data, dst_alpha, 0, height - 1 );

	return ret_image;

}

void fsImage::DeleteICCProfile()
{
	if( m_ICCProfile != NULL ) {
		delete m_ICCProfile;
		m_ICCProfile = NULL;
	}
	m_ICCProfileSize = 0;
}

void fsImage::SetICCProfile( const void *address, size_t size )
{
	DeleteICCProfile();
	if( size > 0 ) {
		m_ICCProfile = new unsigned char[size];
		m_ICCProfileSize = size;
		memcpy( m_ICCProfile, address, size );
	}
}


