/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
This is a wxImage with a better resize algorithm
and with support for ICC color profiles

A note about ICC color profiles
-------------------------------

The image data are reference counted and the profile data are not. I did not have nor the time
nor the will to do it.
As a result profile data could cause problems (memory leaks or crashes) if instances of this class are assigned
to each other. The only safe way to use color profiles data are the following.

READING A PROFILE
-----------------

create the image from file or stream: profile data are loaded if available
immediately copy profile data to another location
immediately call DeleteICProfile()

WRITING A PROFILE
-----------------

set the profile with SetICCProfile()
immediately write the image: the profile will be embedded
immediately call DeleteICProfile()

CONVERSION TO wxImage
---------------------

fsImage has two new variables referring to color profiles: if you follow the guidelines above
the image will not contain any profile.
If the fsImage does not contain any profile it is safe to convert it to a wxImage: the instance will be sliced, losing
profile data. If the fsImage does not contain profile data this is not a problem.

*/

#ifndef _FS_IMAGE_H_
#define _FS_IMAGE_H_

#include "wx/image.h"

class WXDLLEXPORT fsImage: public wxImage
{
public:

    fsImage(){ Init(); }
    fsImage( int width, int height, bool clear = true );
    fsImage( int width, int height, unsigned char* data, bool static_data = false );
    fsImage( int width, int height, unsigned char* data, unsigned char* alpha, bool static_data = false );
    fsImage( const wxString& name, long type = wxBITMAP_TYPE_ANY, int index = -1 );
    fsImage( const wxString& name, const wxString& mimetype, int index = -1 );
    fsImage( const char* const* xpmData );
    virtual ~fsImage();

#if wxUSE_STREAMS
    fsImage( wxInputStream& stream, long type = wxBITMAP_TYPE_ANY, int index = -1 );
    fsImage( wxInputStream& stream, const wxString& mimetype, int index = -1 );
#endif // wxUSE_STREAMS

	// init member variables
	void Init();

	// return the new image with size width*height
    fsImage ScaleFS( int width, int height ) const;

    // rescales the image in place
    fsImage& RescaleFS( int width, int height ) { return *this = ScaleFS(width, height); }

	// return a new image half the size of the original
	fsImage ScaleHalf();
	fsImage& RescaleHalf() { return *this = ScaleHalf(); }

	// return the color profile data
	// this data belongs to this class and must not be deleted by the user
	// if there is no profile the function returns NULL
	const void *GetICCProfileAddress() const { return m_ICCProfile; }
	// return the profile size in bytes
	// if there is no profile the function returns 0
	size_t GetICCProfileSize() const { return m_ICCProfileSize; }
	// set the color profile data
	// memory is allocated by this class and data are copied from the "address" parameter that can be freed after this call
	void SetICCProfile( const void *address, size_t size );
	// delete memory used for the ICC profile
	void DeleteICCProfile();

protected:

	unsigned char *m_ICCProfile;
	size_t m_ICCProfileSize;

private:

    DECLARE_DYNAMIC_CLASS(fsImage)
};



#endif

