/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __METADATA_H__
#define __METADATA_H__

#include "fsdb.h"
#include "utils.h"

#include <exiv2/image.hpp>
#include <exiv2/exif.hpp>

#include <wx/string.h>
#include <wx/datetime.h>

#include <vector>
#include <string>

class CMetadata
{
public:
	CMetadata(void);
	~CMetadata(void);

	// type of metadata handled by the program
	// when adding elements to this enum remember to adjust the lenght of the metadataDescriptions array in the GetTagsNumber() method
	enum MetadataType {
		mtCameraMake = 1,
		mtCameraModel = 2,
		mtImageTimestamp = 3,
		mtExposureTime = 4,
		mtAperture = 5,
		mtExposureBias = 6,
		mtFocalLength = 7,
		mtFocalLenght35mm = 8,
		mtISOSpeed = 9,
		mtExposureMode = 10,
		mtMeteringMode = 11,
		mtImageQuality = 12,
		mtWhiteBalance = 13,
		mtMaxAperture = 14,
		mtFlash = 15,
		mtSoftware = 16
	};

	// color space as read from metadata tags
	enum ColorSpace {
		cs_sRGB,
		csAdobeRGB,
		csUnknown
	};

	// delete all the flags of a file
	// fileID s the primary key of the file row in the database
	static void DeleteFileTags( int fileID, FSDB::Database db );

	// read some basic metadata and return them to the caller
	// those data are not stored in the dabase and are used to add columns to the FILES table
	// return the rotation needed to see the image corretcly, accordingly to the corresponding exif tag
	// return the exif datetime of the image, or an invalid object if not found
	// return width and height of the image as read from metadata, or -1 if not found
	// return the image color space as read from metadata
	static void ReadBasicMetadata( wxString fileName, CUtils::ImageRotation &rotation, wxDateTime &imageDateTime, int &width, int &height, ColorSpace &colorSpace );

	// same as above, but reads the image from a memory buffer
	static void ReadBasicMetadata( const unsigned char *bufferData, long bufferSize, CUtils::ImageRotation &rotation, wxDateTime &imageDateTime, int &width, int &height, ColorSpace &colorSpace );

	// read the tags of a file and store them in the database
	// fileID s the primary key of the file row in the database
	static void ReadFileTags( wxString fileName, int fileID, FSDB::Database db );

	// decode an exif string containing a datatime
	static wxDateTime DecodeTimestamp( std::string exifValue );

	// read the metadata of a file from the database
	void LoadFromDB( int fileID, FSDB::Database db );

	// clear stored data
	void Clear();

	// return the number of metadata tags currently handled
	static int GetTagsNumber() { return 16; }

	// getter methods
	const wxString &GetCameraMake() { return m_CameraMake; }
	const wxString &GetCameraModel() { return m_CameraModel; }
	const wxString &GetSoftware() { return m_Software; }
	const wxString &GetExposureTime() { return m_ExposureTime; }
	const wxString &GetAperture() { return m_Aperture; }
	const wxString &GetMaxAperture() { return m_MaxAperture; }
	const wxString &GetExposureBias() { return m_ExposureBias; }
	const wxString &GetFlash() { return m_Flash; }
	const wxString &GetFocalLength() { return m_FocalLength; }
	const wxString &GetFocalLength35mm() { return m_FocalLength35mm; }
	const wxString &GetISOSpeed() { return m_ISOSpeed; }
	const wxString &GetExposureMode() { return m_ExposureMode; }
	const wxString &GetMeteringMode() { return m_MeteringMode; }
	const wxString &GetImageQuality() { return m_ImageQuality; }
	const wxString &GetWhiteBalance() { return m_WhiteBalance; }
	const wxDateTime &GetImageTimeStamp() { return m_ImageTimestamp; }

private:

	// read a single string tag
	// return an empty string if the tag is not found, or the tag interpreted value if found
	static wxString ReadStringTag( const Exiv2::ExifData& exifData, const wxString &key );

	// read a single long tag
	// return -1 if the key is not found
	static long ReadLongTag( const Exiv2::ExifData& exifData, const wxString &key );

	// set strings in the metadata descriptions data structures
	// cannot be initualized at compile-time because we might have to load translated strings
	void SetMetatadaDescriptions();

	// read some information from metadata
	static void ReadBasicMetadata( Exiv2::Image::AutoPtr image, CUtils::ImageRotation &rotation, wxDateTime &imageDateTime, int &width, int &height, ColorSpace &colorSpace );

	// number of instances of this class in existence
	// since metadata descriptions are static we use this as a flag to set them only once
	static long nItems;

	// array that contains the descriptions of each metadata info
	// useful data starts at index 0
	static wxString *metadataDescriptions;

	// add a metadata row to the database
	static void AppendMetadataRow( FSDB::Recordset rs, int id, int type, wxString value );

	// read a single tag from the current recordset row
	void LoadSingleTagFromDB( FSDB::Recordset rs );

	// metadata values for an item of this class
	// strings will be empty if a tag is missing
	wxString m_CameraMake, m_CameraModel, m_Software, m_ExposureTime, m_Aperture, m_MaxAperture, 
		     m_ExposureBias, m_Flash, m_FocalLength, m_FocalLength35mm, m_ISOSpeed, 
			 m_ExposureMode, m_MeteringMode, m_ImageQuality, m_WhiteBalance;
	wxDateTime m_ImageTimestamp;


};




#endif
