/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
This class is used to encapsulate the creation of SQL strings
so that they can be found in a single place
*/

#ifndef __SQLSTRINGS_H__
#define __SQLSTRINGS_H__

#include "utils.h"

#include <string>

class CSqlStrings
{
public:

	// commands used to create empty recordsets (to insert new rows)
	static std::string SelectVolumesEmpty()
		{ return "SELECT * FROM VOLUMES WHERE VOLUME_ID = -1"; }
	static std::string SelectPathsEmpty()
		{ return "SELECT * FROM PATHS WHERE PATH_ID = -1"; }
	static std::string SelectFilesEmpty()
		{ return "SELECT * FROM FILES WHERE FILE_ID = -1"; }
	static std::string SelectThumbnailsEmpty()
		{ return "SELECT * FROM THUMBNAILS WHERE FILE_ID_TH = -1"; }
	static std::string SelectMetadataEmpty()
		{ return "SELECT * FROM METADATA WHERE FILE_ID_MD = -1"; }
	static std::string SelectPathsFilesEmpty()
		{ return "SELECT * FROM PATHS_FILES WHERE PF_ID = -1"; }

	// select the list of volumes of a certain type ("R" == removable, "F" == fixed)
	static std::string SelectVolumesList( wxString volumeType );
	static std::string SelectVolumesPathID( long volumeID )
		{ return "SELECT PATH_ID FROM PATHS WHERE VOLUME_ID = " + CUtils::long2stdstring(volumeID) + " AND FATHER_ID IS NULL"; }
	static std::string SelectPathsList( long volumeID, long fatherID )
		{ return "SELECT * FROM PATHS WHERE VOLUME_ID = " + CUtils::long2stdstring(volumeID) + " AND FATHER_ID = " + CUtils::long2stdstring(fatherID) + " ORDER BY UPPER(PATH_NAME)"; }

	// return the files in a path
	static std::string SelectFilesInAPath( long pathID )
		{ return "SELECT FILES.*, THUMBNAILS.* FROM FILES INNER JOIN PATHS_FILES ON FILES.FILE_ID = PATHS_FILES.FILE_ID_PF LEFT OUTER JOIN THUMBNAILS ON FILES.FILE_ID = THUMBNAILS.FILE_ID_TH WHERE PATH_ID_PF = " + CUtils::long2stdstring(pathID) + " ORDER BY FILES.IMAGE_DATETIME, UPPER(FILES.FILE_NAME)"; }

	// return the count of files in a path
	static std::string SelectCountFilesInAPath( long pathID )
		{ return "SELECT COUNT(FILES.FILE_ID) AS NROWS FROM FILES INNER JOIN PATHS_FILES ON FILES.FILE_ID = PATHS_FILES.FILE_ID_PF WHERE PATH_ID_PF = " + CUtils::long2stdstring(pathID); }

	static std::string SelectMetadataFileId( long fileID )
		{ return "SELECT * FROM METADATA WHERE FILE_ID_MD = " + CUtils::long2stdstring(fileID); }

	static std::string SelectServiceData()
		{ return "SELECT * FROM SERVICE WHERE SERVICE_ID = 1"; }

	// select the list of vitual paths under a given father ID
	// fatherID == -1 means NULL
	static std::string SelectVirtualPathsList( long fatherID );
		
	// return the virtual files in a virtual path
	static std::string SelectVirtualFilesInAPath( long pathID )
		{ return "SELECT VIRTUAL_FILES.*, FILES.*, THUMBNAILS.* FROM (VIRTUAL_FILES INNER JOIN FILES ON VIRTUAL_FILES.PHYSICAL_FILE_ID = FILES.FILE_ID) LEFT OUTER JOIN THUMBNAILS ON FILES.FILE_ID = THUMBNAILS.FILE_ID_TH WHERE VIRTUAL_FILES.VIRTUAL_PATH_ID = " + CUtils::long2stdstring(pathID) + " ORDER BY FILES.IMAGE_DATETIME, UPPER(FILES.FILE_NAME)"; }

	// return the count of virtual files in a virtual path
	static std::string SelectCountVirtualFilesInAPath( long pathID )
		{ return "SELECT COUNT(VIRTUAL_FILES.VIRTUAL_FILE_ID) AS NROWS FROM VIRTUAL_FILES WHERE VIRTUAL_FILES.VIRTUAL_PATH_ID = " + CUtils::long2stdstring(pathID); }

	// select a volume of a given name and type
	static std::string SelectVolumeByName( wxString volumeName, wxString volumeType );

	// select a path given volumeID, fatherID and path name
	// fatherID == -1 means NULL
	static std::string SelectPathByName( wxString pathName, int volumeID, int fatherID );

	// select a file from its primary key
	static std::string SelectFileFromID( int fileID ) { return "SELECT * FROM FILES WHERE FILE_ID = " + CUtils::long2stdstring(fileID); }

	// select all the virtual paths that contain a physical file
	static std::string SelectVirtualPathsFromPhysicalFileID( int fileID )
		{ return "SELECT * FROM VIRTUAL_FILES WHERE PHYSICAL_FILE_ID = " + CUtils::long2stdstring(fileID) + " ORDER BY VIRTUAL_PATH_ID"; }

	// select used by the search pane
	// text parameters are ignored if == ""
	// date parameters are ignored if not valid
	// the selected folders IDs are == -1 if no folder is selected
	// if a folder is selected the returned query only searches that folder, not the subfolders
	static std::string SelectForSearchPane( wxString fileName, CUtils::TextSearchKind fnSearchKind, 
											wxString extension, wxString description, 
											CUtils::TextSearchKind descrSearchKind, 
											wxDateTime startDate, wxDateTime endDate,
											CUtils::SearchScope sscope,	int selectedFixedFolderID,
											int selectedRemovableFolderID, int selectedVirtualFolderID );

    // select used by the timeline pane
    // looks for different dates where at least more than threshold images where taken
    // return the date and the number of items
    static std::string SelectForTimelinePane( const wxDateTime &startDate, const wxDateTime &endDate, const int threshold );

	// return the files for a given date
	static std::string SelectFilesByDate( const wxDateTime &dt );

	// return the count of files for a given date
	static std::string SelectCountFilesByDate( const wxDateTime &dt );

    // return the full path of a physical folder given data of a file contained in that folder
	static std::string GetFullPhysicalPathFromFile()
		{ return "EXECUTE PROCEDURE SP_GET_FILE_FULL_PATH( ?, ?, ? )"; }

	// return the full path of a physical folder given the ID of the folder itself
	static std::string GetFullPhysicalPath()
		{ return "EXECUTE PROCEDURE SP_GET_FULL_PATH( ?, ? )"; }

	// return the full path of a virtual folder
	static std::string GetFullVirtualPath()
		{ return "EXECUTE PROCEDURE SP_GET_FULL_PATH_VIRTUAL( ?, ? )"; }
		
	// delete a physical volume
	static std::string DeleteVolume()
		{ return "EXECUTE PROCEDURE SP_DELETE_VOLUME( ? )"; }

	// delete a virtual folder
	static std::string DeleteVirtualFolder()
		{ return "EXECUTE PROCEDURE SP_DELETE_VIRTUAL_FOLDER( ? )"; }

	// select the thumbnail of a file
	static std::string SelectFileThumbnail( int fileID )
		{ return "SELECT * FROM THUMBNAILS WHERE FILE_ID_TH = " + CUtils::long2stdstring(fileID); }

	// delete all the metadata for a file
	static std::string DeleteFileMetadata( int fileID )
		{ return "DELETE FROM METADATA WHERE FILE_ID_MD = " + CUtils::long2stdstring(fileID); }

	// delete the thumbnail for a file
	static std::string DeleteFileThumbnail( int fileID )
		{ return "DELETE FROM THUMBNAILS WHERE FILE_ID_TH = " + CUtils::long2stdstring(fileID); }

	// delete the PATHS_FILES entries for a fileID
	static std::string DeleteFilePathsFiles( int fileID )
		{ return "DELETE FROM PATHS_FILES WHERE FILE_ID_PF = " + CUtils::long2stdstring(fileID); }

	// delete a physical folder
	static std::string DeletePhysicalFolder()
		{ return "EXECUTE PROCEDURE SP_DELETE_PHYSICAL_FOLDER( ? )"; }
		
	// delete completely a physical file
	static std::string DeletePhysicalFile()
		{ return "EXECUTE PROCEDURE SP_DELETE_PHYSICAL_FILE( ? )"; }

	// remove a file from the given path. Will also completely remove the file if this is the only path
	static std::string RemovePhysicalFileFromPath()
		{ return "EXECUTE PROCEDURE SP_REMOVE_PHYSFILE_FROM_PATH( ?, ? )"; }
		
	// delete a virtual file
	static std::string DeleteVirtualFile( int fileID )
		{ return "DELETE FROM VIRTUAL_FILES WHERE VIRTUAL_FILE_ID = " + CUtils::long2stdstring(fileID); }
		
	// update the rotation value for a file
	static std::string UpdateFileRotation( int fileID, CUtils::ImageRotation rotation )
		{ return "UPDATE FILES SET ROTATION = " + CUtils::long2stdstring( (long) rotation ) + " WHERE FILE_ID = " + CUtils::long2stdstring(fileID); }

	// update the image datetime field in the FILES table
	static std::string UpdateFileImageDatetime( int fileID, const wxDateTime &dt );

	// update the description of a volume
	static std::string UpdateVolumeDescription( int volumeID, wxString newDescription );

	// update the description of a physical path
	static std::string UpdatePhysicalPathDescription( int pathID, wxString newDescription );

	// update the description of a virtual path
	static std::string UpdateVirtualPathDescription( int pathID, wxString newDescription );

	// update the name of a virtual path
	static std::string UpdateVirtualPathName( int pathID, wxString newName )
	{ return "UPDATE VIRTUAL_PATHS SET PATH_NAME = '" + CUtils::DBwx2std(CUtils::ExpandSingleQuotes(newName)) + "' WHERE PATH_ID = " + CUtils::long2stdstring(pathID); }

	// update the description of a file
	static std::string UpdatePhysicalFileDescription( int fileID, wxString newDescription );

	// create a virtual folder
	static std::string CreateVirtualFolder()
		{ return "EXECUTE PROCEDURE SP_CREATE_UNIQUE_VIRTUALPATH( ?, ?, ? )"; }
		
	// add something to a virtual folder

	// add a file to a virtual path
	static std::string AddPhysicalFileToVirtualPath()
		{ return "EXECUTE PROCEDURE SP_ADD_PHYSFILE_TO_VIRTUALPATH( ?, ? )"; }

	// append a volume to a virtual path
	static std::string AppendVolumeToVirtualPath()
		{ return "EXECUTE PROCEDURE SP_ADD_VOLUME_TO_VIRTUALPATH( ?, ?, ? )"; }

	// append a physical to a virtual path
	static std::string AppendPhysicalPathToVirtualPath()
		{ return "EXECUTE PROCEDURE SP_APPEND_PHYSPTH_TO_VIRTUALPTH( ?, ? )"; }

private:

	// escape wildcards for a firebird database
	// we don't want that "_" and "%" are interpreted as wildcards
	static wxString EscapeWildcards( wxString txt, wxString escape );

};



#endif

