/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <wx/stdpaths.h>
#include <wx/filename.h>
#include "utils.h"

wxString CUtils::applicationName = wxT("vvvP");
wxString CUtils::applicationVersion = wxT("1.1");
int CUtils::expectedDatabaseVersion = 1;
wxString CUtils::strucUpdateDbName = wxT("vvvp-struct-update.fdb");

//-----------------------------------------------------------------------------
//! converts wxString to std::string
std::string CUtils::wx2std(const wxString& input, wxMBConv* conv)
{
    if (input.empty())
        return "";
    if (!conv)
        conv = wxConvCurrent;
    const wxWX2MBbuf buf(input.mb_str(*conv));
    // conversion may fail and return 0, which isn't a safe value to pass 
    // to std:string constructor
    if (!buf)
        return "";
    return std::string(buf);
}
//-----------------------------------------------------------------------------
//! converts std:string to wxString
wxString CUtils::std2wx(const std::string& input, wxMBConv* conv)
{
    if (input.empty())
        return wxEmptyString;
    if (!conv)
        conv = wxConvCurrent;
    return wxString(input.c_str(), *conv);
}


// string conversion for database access
// the database uses UTF8 encoding
std::string CUtils::DBwx2std( const wxString& input ) {
	std::string s = wx2std( input, &wxConvUTF8 );
	return wx2std( input, &wxConvUTF8 );
}
wxString CUtils::DBstd2wx( const std::string& input ) {
	return std2wx( input, &wxConvUTF8 );
}


FSDB::Timestamp CUtils::dt_wx2fsdb( const wxDateTime& dt ) {
	return FSDB::Timestamp( dt.GetYear(), ((int) dt.GetMonth()) + 1, dt.GetDay(), dt.GetHour(), dt.GetMinute(), dt.GetSecond() );
}

wxDateTime CUtils::dt_fsdb2wx( const FSDB::Timestamp& ts ) {
	return wxDateTime( ts.Day(), (wxDateTime::Month) (ts.Month() - 1), ts.Year(), ts.Hours(), ts.Minutes(), ts.Seconds() );
}

int64_t CUtils::i64_wx2fsdb( const wxULongLong& i ) {
	long hi = i.GetHi();
	unsigned long lo = i.GetLo();
	return (((int64_t) hi) << 32) + lo;
}

wxULongLong CUtils::i64_fsdb2wx( const int64_t i ) {
	unsigned long lo = (unsigned long) i;
	long hi = (long) (i >> 32);
	return wxULongLong( hi, lo );
}


void CUtils::MsgErr( wxString errMsg ){
	wxMessageDialog dialog( NULL, errMsg, applicationName, wxOK|wxICON_ERROR );
	dialog.ShowModal();
}

void CUtils::MsgInfo( wxString infoMsg ){
	wxMessageDialog dialog( NULL, infoMsg, applicationName, wxOK|wxICON_INFORMATION );
	dialog.ShowModal();
}

bool CUtils::MsgAskNo( wxString msg ){
	wxMessageDialog dialog( NULL, msg, applicationName, wxYES_NO|wxNO_DEFAULT|wxICON_EXCLAMATION );
	return dialog.ShowModal() == wxID_YES;
}

bool CUtils::MsgAskYes( wxString msg ){
	wxMessageDialog dialog( NULL, msg, applicationName, wxYES_NO|wxYES_DEFAULT|wxICON_EXCLAMATION );
	return dialog.ShowModal() == wxID_YES;
}


wxString CUtils::HumanReadableFileSize( wxULongLong size ) {
	wxString retVal;

	if( size > 1024*1024 ) {
		size = size / (1024*1024);
		retVal = size.ToString() + wxT(" MB");
	}
	else {
		if( size > 1024 ) {
			size = size / 1024;
			retVal = size.ToString() + wxT(" KB");
		}
		else {
			retVal = size.ToString();
		}
	}

	return retVal;
}


wxString CUtils::GetApplicationName(void) {
	return applicationName;
}

wxString CUtils::GetApplicationVersion(void) {
	return applicationVersion;
}

int CUtils::GetExpectedDatabaseVersion(void) {
	return expectedDatabaseVersion;
}

wxString CUtils::GetStructUpdateDbName(void) {
	wxString appPath = wxStandardPaths::Get().GetExecutablePath();
	wxFileName fn( appPath );
	fn.SetFullName( strucUpdateDbName );
	wxString fullName = fn.GetFullPath();
	return fullName;
}


// converts a long to a string
wxString CUtils::long2string( long val, unsigned int nPos ) {
	wxString retVal;
	retVal.Printf( wxT("%ld"), val );
	if( nPos > 0 && retVal.length() < nPos ) {
		int nChars = nPos - retVal.length();
		wxString zeroes( wxT("0"), nChars );
		retVal = zeroes + retVal;
	}

	return retVal;
}

std::string CUtils::long2stdstring( long val, unsigned int nPos ) {
	wxString s = long2string( val, nPos );
	return wx2std( s );
}

wxString CUtils::Encrypt( wxString s ) {
	wxString key = "w2m5t7i0g4ndnra5s4lfdpvlyrepfjemwksdjfrpwuanrtogbjentruglk60947235128";
	wxString sout( ' ', s.Len() );
	size_t k;

	for( k = 0; k < s.Len(); k++ ) {
		char ch = static_cast<char>(s[k]) ^ static_cast<char>(key[k % key.Len()]);
		sout[k] = ch;
	}

	return sout;
}

wxString CUtils::GetHelpFileName(void) {
	wxString appPath = wxStandardPaths::Get().GetExecutablePath();
	wxFileName fn( appPath );
	fn.AppendDir( wxT("help") );
	fn.AppendDir( wxT("en") );
	fn.SetFullName( wxT("vvvP") );
	wxString fullName = fn.GetFullPath();
	return fullName;
}

CUtils::ImageRotation CUtils::GetNewImageRotation( CUtils::ImageRotation oldRotation, CUtils::ImageRotation change ) {

	int rtmp = (int) oldRotation + (int) change;
	while( rtmp > 3 ) {
		rtmp = rtmp - 4;
	}

	wxASSERT( rtmp >= 0 && rtmp < 4 );

	CUtils::ImageRotation newRotation = (CUtils::ImageRotation) rtmp;
	return newRotation;
}

wxImage CUtils::RotateImage( wxImage inputImage, CUtils::ImageRotation rotation ) {

	wxImage outputImage;
	switch( rotation ) {
		case CUtils::irNoRotation:
			outputImage = inputImage;
			break;
		case CUtils::irRotateRight:
			outputImage = inputImage.Rotate90( true );
			break;
		case CUtils::irUpsideDown:
			outputImage = inputImage.Rotate90( true );
			outputImage = outputImage.Rotate90( true );
			break;
		case CUtils::irRotateLeft:
			outputImage = inputImage.Rotate90( false );
			break;
	}
	return outputImage;
}

wxString CUtils::ExpandSingleQuotes( wxString strIn ) {

	strIn.Replace( wxT("'"), wxT("''"), true );
	return strIn;

}

wxString CUtils::NewLine2Space( const wxString& objectDescription ) {
	wxString retVal;

	retVal = objectDescription;
	if( !retVal.empty() ) {
		retVal.Replace( wxT("\n"), wxT(" ") );
	}
	return retVal;
}

wxBitmap CUtils::EnlargeBitmap( wxBitmap bmpIn, int w, int h )
{
	wxImage img = bmpIn.ConvertToImage();
	int wOrg = img.GetWidth();
	int hOrg = img.GetHeight();

	wxASSERT( w > wOrg );
	wxASSERT( h > hOrg );

	wxSize size( w, h );
	wxPoint pos( (w - wOrg)/2, (h - hOrg)/2 );	// center original bitmap in the new image
	img.Resize( size, pos );
	return wxBitmap( img );
}

wxString CUtils::GetDateFormatString()
{
    wxString fmt = wxLocale::GetInfo(wxLOCALE_SHORT_DATE_FMT);
    fmt.Replace("%y", "%Y");
    return fmt;
}


