/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __UTILS_H__
#define __UTILS_H__

#include "fsdb.h"

#include <wx/string.h>
#include <wx/msgdlg.h>
#include <wx/image.h>
#include <wx/bitmap.h>
#include <string>

class wxMBConv;

class CUtils
{
public:

	// possible rotations of an image
	enum ImageRotation {
		irNoRotation = 0,
		irRotateRight = 1,
		irUpsideDown = 2,
		irRotateLeft = 3
	};

	// possible choices for text name search
	enum TextSearchKind {
		tskIsEqual = 0,
		tskStartsWith,
		tskContains
	};

	// possible search choices
	enum SearchScope {
		ssAllFiles = 0,
		ssSelectedFixedLocationFolder,
		ssSelectedRemovableLocationFolder,
		ssSelectedVirtualFolder
	};

	// computes the new value for an image rotation
	// oldROtation is the original rotation value
	// change is the rotation to apply to the original one
	// return the new rotation
	static ImageRotation GetNewImageRotation( ImageRotation oldRotation, ImageRotation change );

	// return a rotated copy of an image
	static wxImage RotateImage( wxImage inputImage, CUtils::ImageRotation rotation );

	// functions used to convert between std::string e wxString.
	// code "borrowed" from the FlameRobin source code
	static std::string wx2std(const wxString& input, wxMBConv* conv = wxConvCurrent);
	static wxString std2wx(const std::string& input, wxMBConv* conv = wxConvCurrent);

	// functions used to convert between wxString and std::string,
	// used to interface with the database which requires a particular, fixed encodiding
	static std::string DBwx2std(const wxString& input);
	static wxString DBstd2wx(const std::string& input);

	// convert a wxWidgets datetime object to the equivalent used by FSDB
	static FSDB::Timestamp dt_wx2fsdb( const wxDateTime& dt );
	
	// convert an FSDB datetime object to the equivalent used by wxWidgets
	static wxDateTime dt_fsdb2wx( const FSDB::Timestamp& ts );

	// convert a wxWidgets 64 bit unsighed integer to an int64_t
	static int64_t i64_wx2fsdb( const wxULongLong& i );

	// convert an int64_t to a wxWidgets 64 bit unsigned integer
	static wxULongLong i64_fsdb2wx( const int64_t i );

	// return the size of a file in human readable format (KB, MB and so on...)
	static wxString HumanReadableFileSize( wxULongLong size );

	// displays an error message
	static void MsgErr( wxString errMsg );

	// displays an information message
	static void MsgInfo( wxString infoMsg );

	// ask a question to the user. possible answers: yes or no (default)
	// returns true if the user has chosen yes
	static bool MsgAskNo( wxString msg );

	// ask a question to the user. possible answers: yes (default) or no
	// returns true if the user has chosen yes
	static bool MsgAskYes( wxString msg );

	// returns the application name
	static wxString GetApplicationName(void);

	// returns the application version
	static wxString GetApplicationVersion(void);

	// returns the expected database version
	static int GetExpectedDatabaseVersion(void);

	// converts a long to a wxString
	// nPos is the minimum length of the string, left padded with 0s
	static wxString long2string( long val, unsigned int nPos = 0 );

	// converts a long to a std::string
	// nPos is the minimum length of the string, left padded with 0s
	static std::string long2stdstring( long val, unsigned int nPos = 0 );

	// returns the name of the database containing the catalog structure changes, used to dynamically update a catalog
	static wxString GetStructUpdateDbName(void);

	// encrypts/decrypts a string with a simple XOR
	static wxString Encrypt( wxString s );

	// returns the name of the help file
	static wxString GetHelpFileName();

	// return a string where single quotes chars have been doubled ( "O'Hara" --> "O''Hara" )
	// used in sql string
	static wxString ExpandSingleQuotes( wxString strIn );

	// change newlines in a string to spaces
	static wxString NewLine2Space( const wxString& objectDescription );

	// return an enlarged version of a bitmap
	// w and h are the dimensions of the new returned
	static wxBitmap EnlargeBitmap( wxBitmap bmpIn, int w, int h );

    // return the date format string used by this computer
    static wxString GetDateFormatString();

private:
	// name of the current application
	static wxString applicationName;

	// current application version
	static wxString applicationVersion;

	// the current version of the application expects this database version
	static int expectedDatabaseVersion;

	// name of the database that contains the database changes from version to version
	static wxString strucUpdateDbName;

};



#endif
